using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace ProjectLineCounter
{
    public class CounterOptions
    {
        private bool _includeBracket;
        private bool _inclueUsingBlocks;
        private List<string> _specialRegions;
        private bool _includeDocument;
        private bool _includeBlanks;


        internal CounterOptions()
        {
            this._specialRegions = new List<string>();
            this._includeBracket = false;
            this._inclueUsingBlocks = false;
            this._includeDocument = false;
            this._includeBlanks = false;
        }

        /// <summary>
        /// Gets or sets whether the blank lines will be added into result.
        /// </summary>
        public bool IncludeBlanks
        {
            get { return _includeBlanks; }
            set { _includeBlanks = value; }
        }

        /// <summary>
        /// Gets or sets whether the document lines written in source code
        /// will be added into result.
        /// </summary>
        public bool IncludeDocument
        {
            get { return _includeDocument; }
            set { _includeDocument = value; }
        }

        /// <summary>
        /// The regions that will be omitted from countting.
        /// </summary>
        public List<string> SpecialRegions
        {
            get { return _specialRegions; }
        }

        /// <summary>
        /// Gets or sets whether using block lines should be added into result.
        /// </summary>
        public bool InclueUsingBlocks
        {
            get { return _inclueUsingBlocks; }
            set { _inclueUsingBlocks = value; }
        }

        /// <summary>
        /// Gets or sets whether the line only contains single bracket
        /// ("{" or "}") should be added into result.
        /// </summary>
        public bool InclueBracket
        {
            get { return _includeBracket; }
            set { _includeBracket = value; }
        }
    }

    internal class LineCounter
    {
        private enum BlockType
        {
            Region,
            Using,
            Method,
            TypeDefination,
            Document
        }

        internal static CounterResult CountLines(string fileName, CounterOptions options)
        {
            CounterResult result;
            long lines = 0;
            long length = 0;
            string name;
            StreamReader reader = null;

            FileInfo info = new FileInfo(fileName);

            name = info.Name;

            try
            {
                reader = info.OpenText();

                length = reader.BaseStream.Length;
                string line = reader.ReadLine().Trim();

                while (line != null)
                {
                    if (line.IndexOf("using ") != -1 && line.IndexOf(";") != -1 && line.IndexOf("{") == -1 && !options.InclueUsingBlocks)
                    {
                        line = GoToEndOfBlock(reader, BlockType.Using);
                    }
                    else if (line.IndexOf("#region") != -1 && options.SpecialRegions.Contains(GetRegionName(line)))
                    {
                        line = GoToEndOfBlock(reader, BlockType.Region);
                    }
                    else if (line == "" && !options.IncludeBlanks)
                    {
                        line = GetNextLine(reader);
                    }
                    else if ((line == "{" || line == "}") && !options.InclueBracket)
                    {
                        line = GetNextLine(reader);
                    }
                    else
                    {
                        lines++;
                        line = GetNextLine(reader);
                    }
                }
            }
            catch { }
            finally
            {
                if (reader != null)
                {
                    reader.Close();
                    reader.Dispose();
                }
            }

            result = new CounterResult(length, lines, name, fileName);

            return result;
        }
        internal static CounterResult CountLines(string fileName)
        {
            return CountLines(fileName, new CounterOptions());
        }

        private static string GetNextLine(StreamReader reader)
        {
            string line;

            if (reader.EndOfStream)
            {
                line = null;
            }
            else
            {
                line = reader.ReadLine().Trim();
            }

            return line;
        }

        private static string GetRegionName(string line)
        {
            string result = line.Trim().Replace("#region ", string.Empty);

            return result;
        }

        private static string GoToEndOfBlock(StreamReader reader, BlockType type)
        {
            string line;
            bool toContinue = true;
            string firstLine = null;

            switch (type)
            {
                case BlockType.Document:
                    {
                        while (toContinue && !reader.EndOfStream)
                        {
                            line = GetNextLine(reader);
                            if (line.TrimStart(' ').IndexOf("///") != 0)
                            {
                                toContinue = false;
                                firstLine = line;
                            }
                        }
                        break;
                    }
                case BlockType.Method:
                    {
                        break;
                    }
                case BlockType.Region:
                    {
                        int regions = 1;
                        line = GetNextLine(reader);

                        while (regions > 0 && !reader.EndOfStream)
                        {
                            line = GetNextLine(reader);
                            if (line.IndexOf("#endregion") == 0)
                            {
                                regions--;
                            }
                            else if (line.IndexOf("#region") == 0)
                            {
                                regions++;
                            }
                        }

                        firstLine = GetNextLine(reader);

                        break;
                    }
                case BlockType.TypeDefination:
                    {
                        break;
                    }
                case BlockType.Using:
                    {
                        while (toContinue && !reader.EndOfStream)
                        {
                            line = GetNextLine(reader);

                            if (line.Trim().IndexOf("using ") == -1)
                            {
                                toContinue = false;
                                firstLine = line;
                            }
                        }
                        break;
                    }
            }

            return firstLine;
        }
    }
}