using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace ProjectLineCounter
{
    internal partial class MainForm : Form
    {
        private ApplicationSettings settings;
        private OptionsForm optionsForm;
        private AboutBox aboutBox;
        private FileNameCollection filesAdded;
        private bool settingsChanged;


        internal MainForm()
        {
            settings = Util.LoadSettings();
            optionsForm = new OptionsForm(settings);
            filesAdded = new FileNameCollection();
            aboutBox = new AboutBox();
            this.FormClosed += new FormClosedEventHandler(MainForm_FormClosed);

            InitializeComponent();
        }

        #region Operations

        private void AddFile(string file)
        {
            AddFiles(new FileNameCollection(file), false);
        }

        private void AddFolder(string folder, bool addSubfolders)
        {
            FileNameCollection files = Util.GetFiles(folder, addSubfolders, settings.FilePatterns, settings.ExcludeFilePatterns);

            AddFiles(files, false);
        }

        private void AddFiles(FileNameCollection files, bool recount)
        {
            foreach (string file in files)
            {
                if (!recount)
                {
                    filesAdded.Add(file);
                }
                CounterResult result = LineCounter.CountLines(file, settings.CounterOptions);
                listView_FileList.Items.Add(new FileListItem(result));
            }
        }

        private void SumFiles(System.Collections.IList items)
        {
            long sum_Line = 0;
            long sum_Length = 0;
            int fileCount = 0;

            foreach (FileListItem item in items)
            {
                sum_Line += item.Result.Lines;
                sum_Length += item.Result.Length;
                fileCount++;
            }

            string message = string.Format("Files:\t{0}\nLength:\t{1} bytes\nLines:\t{2}", fileCount, sum_Length, sum_Line);
            MessageBox.Show(message, "Sum Result", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        private void Recount()
        {
            listView_FileList.Items.Clear();

            AddFiles(filesAdded, true);
        }

        private void SelectAll()
        {
            foreach (ListViewItem item in listView_FileList.Items)
            {
                item.Selected = true;
            }
        }

        private void RemoveSelectedFiles()
        {
            ListViewItem[] items = new ListViewItem[listView_FileList.SelectedItems.Count];
            listView_FileList.SelectedItems.CopyTo(items, 0);

            foreach (FileListItem item in items)
            {
                filesAdded.Remove(item.Result.FullName);
                listView_FileList.Items.Remove(item);
            }
        }

        private void RemoveAllFiles()
        {
            filesAdded.Clear();
            listView_FileList.Items.Clear();
        }

        private static void Exit()
        {
            Application.Exit();
        }

        #endregion

        #region Event Handlers

        private void recountToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Recount();
        }

        private void MainForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            if (settingsChanged)
            {
                Util.SaveSettings(settings);
            }
        }

        private void listView_FileList_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.FileDrop))
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }

        private void listView_FileList_DragDrop(object sender, DragEventArgs e)
        {
            foreach (string value in (string[])e.Data.GetData(DataFormats.FileDrop))
            {
                DropValueType type = Util.GetDropValueType(value);

                switch (type)
                {
                    case DropValueType.File:
                        AddFile(value);
                        break;
                    case DropValueType.Folder:
                        AddFolder(value, settings.SearchSubfolders);
                        break;
                }
            }
        }

        private void addFileToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (openFileDialog.ShowDialog() == DialogResult.OK)
            {
                foreach (string file in openFileDialog.FileNames)
                {
                    AddFile(file);
                }

                openFileDialog.Reset();
            }
        }

        private void addFolderToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (folderBrowserDialog.ShowDialog() == DialogResult.OK)
            {
                AddFolder(folderBrowserDialog.SelectedPath, settings.SearchSubfolders);

                folderBrowserDialog.SelectedPath = string.Empty;
            }
        }

        private void optionsToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            if (optionsForm.ShowDialog() == DialogResult.OK)
            {
                settings = optionsForm.Settings;
                settingsChanged = true;

                MessageBox.Show("Changes will take effect when you recount or add new files.", "Options", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
        }

        private void button_Recount_Click(object sender, EventArgs e)
        {
            Recount();
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            aboutBox.ShowDialog();
        }

        private void clearFilesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            RemoveAllFiles();
        }

        private void removeSelectedFilesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            RemoveSelectedFiles();
        }

        private void sumSelectedFilesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SumFiles(listView_FileList.SelectedItems);
        }

        private void listView_FileList_ItemSelectionChanged(object sender, ListViewItemSelectionChangedEventArgs e)
        {
            if (listView_FileList.SelectedItems.Count == 0)
            {
                removeSelectedFilesToolStripMenuItem.Enabled = false;
                sumSelectedFilesToolStripMenuItem.Enabled = false;
                removeSelectedFilesToolStripMenuItem_MainMenu.Enabled = false;
                sumSelectedFilesToolStripMenuItem_MainMenu.Enabled = false;
            }
            else
            {
                removeSelectedFilesToolStripMenuItem.Enabled = true;
                sumSelectedFilesToolStripMenuItem.Enabled = true;
                removeSelectedFilesToolStripMenuItem_MainMenu.Enabled = true;
                sumSelectedFilesToolStripMenuItem_MainMenu.Enabled = true;
            }
        }

        private void sumAllFilesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SumFiles(listView_FileList.Items);
        }

        private void selectAllToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SelectAll();
        }

        private void listView_FileList_ItemActivate(object sender, EventArgs e)
        {
            //FileListItem item = listView_FileList.SelectedItems[0];
        }

        private void listView_FileList_KeyDown(object sender, KeyEventArgs e)
        {
            switch (e.KeyCode)
            {
                case Keys.Delete:
                    {
                        removeSelectedFilesToolStripMenuItem.PerformClick();
                        break;
                    }
                case Keys.A:
                    {
                        if (e.Control)
                        {
                            selectAllToolStripMenuItem.PerformClick();
                        }
                        break;
                    }
            }
        }

        private void selectAllToolStripMenuItem_MainMenu_Click(object sender, EventArgs e)
        {
            SelectAll();
        }

        private void sumSelectedFilesToolStripMenuItem_MainMenu_Click(object sender, EventArgs e)
        {
            SumFiles(listView_FileList.SelectedItems);
        }

        private void sumAllFilesToolStripMenuItem_MainMenu_Click(object sender, EventArgs e)
        {
            SumFiles(listView_FileList.Items);
        }

        private void removeSelectedFilesToolStripMenuItem_MainMenu_Click(object sender, EventArgs e)
        {
            RemoveSelectedFiles();
        }

        private void removeAllFilesToolStripMenuItem_MainMenu_Click(object sender, EventArgs e)
        {
            RemoveAllFiles();
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Exit();
        }

        #endregion
    }
}