using System;
using System.Collections.Generic;
using System.Text;

namespace LineCounter.Application
{
    internal static class ApplicationRuntimeSettings
    {
        public static event EventHandler ShowAllFilesInSubfolderChanged;
        public static event EventHandler ShowFileIconChanged;
        public static event EventHandler FileContentFilterChanged;
        public static event EventHandler SettingsReseted;

 
        private static Nullable<FileContentType> _fileContentFilter;


        static ApplicationRuntimeSettings()
        {
            LineCounter.Application.Properties.Settings.Default.PropertyChanged += new System.ComponentModel.PropertyChangedEventHandler(DefaultSettings_PropertyChanged);
            _fileContentFilter = null;
        }


        /// <summary>
        /// Gets the default file search pattern collection.
        /// </summary>
        private static SearchPatternCollection DefaultFileSearchPatterns
        {
            get
            {
                return new SearchPatternCollection("*.Designer.*", "AssemblyInfo.*", "Resources.resx", "Settings.settings");
            }
        }

        /// <summary>
        /// Gets the default folder search pattern collection.
        /// </summary>
        private static SearchPatternCollection DefaultFolderSearchPatterns
        {
            get
            {
                return new SearchPatternCollection("bin", "obj", "Properties", "Release", "Debug");
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether subfolders should be searched when importing files or folders.
        /// </summary>
        public static Boolean SearchSubfolders
        {
            get { return LineCounter.Application.Properties.Settings.Default.SearchSubfolders; }
            set { LineCounter.Application.Properties.Settings.Default.SearchSubfolders = value; }
        }

        public static SearchPatternCollection FileSearchPatterns
        {
            get 
            {
                SearchPatternCollection result = null;

                if (LineCounter.Application.Properties.Settings.Default.FileSearchPatterns == null)
                {
                    result = DefaultFileSearchPatterns;
                }
                else
                {
                    result = LineCounter.Application.Properties.Settings.Default.FileSearchPatterns;
                }

                return result;
            }
            set { LineCounter.Application.Properties.Settings.Default.FileSearchPatterns = value; }
        }

        public static SearchPatternCollection FolderSearchPatterns
        {
            get
            {
                SearchPatternCollection result = null;

                if (LineCounter.Application.Properties.Settings.Default.FolderSearchPatterns == null)
                {
                    result = DefaultFolderSearchPatterns;
                }
                else
                {
                    result = LineCounter.Application.Properties.Settings.Default.FolderSearchPatterns;
                }

                return result;
            }
            set { LineCounter.Application.Properties.Settings.Default.FolderSearchPatterns = value; }
        }

        public static SearchFilterType FileSearchFilterType
        {
            get { return LineCounter.Application.Properties.Settings.Default.FileSearchFilterType; }
            set { LineCounter.Application.Properties.Settings.Default.FileSearchFilterType = value; }
        }

        public static SearchFilterType FolderSearchFilterType
        {
            get { return LineCounter.Application.Properties.Settings.Default.FolderSearchFilterType; }
            set { LineCounter.Application.Properties.Settings.Default.FolderSearchFilterType = value; }
        }

        public static String CountersFolder
        {
            get { return LineCounter.Application.Properties.Settings.Default.CountersFolder; }
        }

        public static Boolean ShowFileIcon
        {
            get { return LineCounter.Application.Properties.Settings.Default.ShowFileIcon; }
            set { LineCounter.Application.Properties.Settings.Default.ShowFileIcon = value; }
        }

        public static Boolean ShowAllFilesInSubfolder
        {
            get { return LineCounter.Application.Properties.Settings.Default.ShowAllFilesInSubfolder; }
            set { LineCounter.Application.Properties.Settings.Default.ShowAllFilesInSubfolder = value; }
        }

        public static ICollection<String> SupportedExtensions
        {
            get { return CounterManager.Current.SupportedExtensions; }
        }

        public static Nullable<FileContentType> FileContentFilter
        {
            get { return _fileContentFilter; }
            set
            { 
                _fileContentFilter = value;

                OnFileContentFilterChanged();
            }
        }

        public static Boolean EnableFileContentFilter
        {
            get { return FileContentFilter.HasValue; }
            set
            {
                if (value == true)
                {
                    if (FileContentFilter == null)
                    {
                        FileContentFilter = FileContentType.Countable;
                    }
                }
                else
                {
                    FileContentFilter = null;
                }
            }
        }

        public static Boolean ImportIgnoredFiles
        {
            get { return LineCounter.Application.Properties.Settings.Default.ImportIgnoredFiles; }
            set { LineCounter.Application.Properties.Settings.Default.ImportIgnoredFiles = value; }
        }

        public static Boolean ImportUnknownFiles
        {
            get { return LineCounter.Application.Properties.Settings.Default.ImportUnknownFiles; }
            set { LineCounter.Application.Properties.Settings.Default.ImportUnknownFiles = value; }
        }

        public static String ProjectUrl
        {
            get { return Properties.Settings.Default.ProjectUrl; }
        }


        /// <summary>
        /// Saves all storable settings to file.
        /// </summary>
        public static void SaveStorableSettings()
        {
            LineCounter.Application.Properties.Settings.Default.Save();

            // Save counter options
            foreach (CounterItem counter in CounterManager.Current.CounterItems)
            {
                if (counter.HasEditableOptions)
                {
                    counter.PropertyDescriptor.SaveOptions();
                }
            }
        }

        public static void ResetAllSettings()
        {
            LineCounter.Application.Properties.Settings.Default.Reset();

            // Reset counter options
            foreach (CounterItem counter in CounterManager.Current.CounterItems)
            {
                if (counter.HasEditableOptions)
                {
                    counter.PropertyDescriptor.ResetOptions();
                }
            }

            OnSettingsReseted();
        }


        private static void DefaultSettings_PropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
            switch (e.PropertyName)
            {
                case "ShowAllFilesInSubfolder":
                    {
                        OnShowAllFilesInSubfolderChanged();
                        break;
                    }
                case "ShowFileIcon":
                    {
                        OnShowFileIconChanged();
                        break;
                    }
            }
        }

        private static void OnShowAllFilesInSubfolderChanged()
        {
            if (ShowAllFilesInSubfolderChanged != null)
            {
                ShowAllFilesInSubfolderChanged(LineCounter.Application.Properties.Settings.Default, EventArgs.Empty);
            }
        }

        private static void OnShowFileIconChanged()
        {
            if (ShowFileIconChanged != null)
            {
                ShowFileIconChanged(LineCounter.Application.Properties.Settings.Default, EventArgs.Empty);
            }
        }

        private static void OnFileContentFilterChanged()
        {
            if (FileContentFilterChanged != null)
            {
                FileContentFilterChanged(null, EventArgs.Empty);
            }
        }

        private static void OnSettingsReseted()
        {
            if (SettingsReseted != null)
            {
                SettingsReseted(null, EventArgs.Empty);
            }
        }
    }
}
