using System;
using System.Collections;
using System.Collections.Generic;
//===============================================================================
// SPECIAL THANKS TO SCOTT CORN
// WHO TAUGHT ME HOW TO WRITE THE COMMAND CLASSES 
//===============================================================================

using System.Text;
using System.Windows.Forms;
using System.Drawing;
using System.ComponentModel;

namespace LineCounter.Application
{
    /// <summary>
    /// Manages commands and the UI eliments that are linked to them.
    /// </summary>
    public class CommandManager : Component
    {
        private class CommandActionDictionary : Dictionary<String, List<LineCounter.Application.CommandManager.CommandAction>> 
        {
            public void Add(String identifier, CommandAction action)
            {
                List<LineCounter.Application.CommandManager.CommandAction> actionList = null;

                if (!this.TryGetValue(identifier, out actionList))
                {
                    actionList = new List<LineCounter.Application.CommandManager.CommandAction>();
                    this.Add(identifier, actionList);
                }

                actionList.Add(action);
            }

            public void Remove(String identifier, CommandAction action)
            {
                List<LineCounter.Application.CommandManager.CommandAction> actionList = null;

                if (this.TryGetValue(identifier, out actionList))
                {
                    actionList.Remove(action);
                }
            }

            public IEnumerable<CommandAction> GetCommandActions(String identifier)
            {
                List<LineCounter.Application.CommandManager.CommandAction> actionList = null;

                if (!this.TryGetValue(identifier, out actionList))
                {
                    actionList = new List<LineCounter.Application.CommandManager.CommandAction>();
                }

                return actionList;
            }
        }

        private enum CommandAction
        {
            Disable = 0,
            Enable,
            Hide,
            Show,
            Check,
            Uncheck,
            Text,
            ChangeLargeImage,
            ChangeSmallImage
        }

        #region Event Declearations

        /// <summary>
        /// Occurs when a command is executed.
        /// </summary>
        public event EventHandler<CommandManagerEventArgs> CommandExecuted;

        /// <summary>
        /// Occurs when a command item's enabled state changed.
        /// </summary>
        public event EventHandler<CommandManagerEventArgs> CommandEnabledChanged;

        /// <summary>
        /// Occurs when a command item's visible state changed.
        /// </summary>
        public event EventHandler<CommandManagerEventArgs> CommandVisibleChanged;

        /// <summary>
        /// Occurs when a command item's checked state changed.
        /// </summary>
        public event EventHandler<CommandManagerEventArgs> CommandCheckedChanged;

        /// <summary>
        /// Occurs when a command item's large image key is changed.
        /// </summary>
        public event EventHandler<CommandManagerEventArgs> CommandLargeImageKeyChanged;

        /// <summary>
        /// Occurs when a command item's small image key is changed.
        /// </summary>
        public event EventHandler<CommandManagerEventArgs> CommandSmallImageKeyChanged;

        public event EventHandler<CommandManagerEventArgs> CommandTextChanged;

        public event EventHandler<CommandManagerEventArgs> CommandToolStripItemDisplayStleChanged;

        public event EventHandler SmallImageListChanged;

        public event EventHandler LargeImageListChanged;


        #endregion

        #region Local Varalables

        private CommandDictionary _commandItems;
        private bool _disposed;
        private ImageList _smallImageList;
        private ImageList _largeImageList;

        #endregion

        #region Properties

        public CommandDictionary CommandItems
        {
            get { return _commandItems; }
        }

        public ImageList LargeImageList
        {
            get { return _largeImageList; }
            set
            {
                if (_largeImageList != value)
                {
                    _largeImageList = value;
                    OnLargeImageListChanged();
                }
            }
        }

        public ImageList SmallImageList
        {
            get { return _smallImageList; }
            set
            {
                if (_smallImageList != value)
                {
                    _smallImageList = value;
                    OnSmallImageListChanged();
                }
            }
        }

        /// <summary>
        /// Gets the current command item count.
        /// </summary>
        //[Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public int CommandItemCount
        {
            get
            {
                return this._commandItems.Count;

            }

        }

        #endregion

        #region Constractor & Distractor

        /// <summary>
        /// Initializes a new instance of the CommandManager class.
        /// </summary>
        public CommandManager()
        {
            this._commandItems = new CommandDictionary();
        }

        /// <summary>
        /// Releases the unmanaged resources used by the CommandManger and optionally releases the managed resources.
        /// </summary>
        /// <param name="disposing">true to release both managed and unmanaged resources; false to release only unmanaged resources.</param>
        protected override void Dispose(System.Boolean disposing)
        {
            try
            {
                this._disposed = true;
            }
            catch
            {
                throw;
            }
            finally
            {
                base.Dispose(disposing);
            }
        }

        #endregion

        #region Public Operations

        /// <summary>
        /// Returns if the item(s) associated with the passed command key is enabled.
        /// </summary>
        /// <param name="commandKey">The key of the command.</param>
        /// <value>true or false</value>
        public bool IsCommandEnabled(string commandIdentifier)
        {
            if (commandIdentifier == null)
            {
                throw new ArgumentNullException("commandIdentifier");
            }

            Command command = null;

            if (CommandItems.TryGetValue(commandIdentifier, out command))
            {
                return command.Enable;
            }

            return false;
        }

        /// <summary>
        /// Returns if the item(s) associated with the passed command key is visible.
        /// </summary>
        /// <param name="commandKey">The key of the command.</param>
        /// <value>true or false</value>
        public bool IsCommandVisible(string commandIdentifier)
        {
            if (commandIdentifier == null)
            {
                throw new ArgumentNullException("commandIdentifier");
            }

            Command command = null;

            if (CommandItems.TryGetValue(commandIdentifier, out command))
            {
                return command.Visible;
            }

            return false;
        }

        /// <summary>
        /// Returns if the item(s) associated with the passed command key is checked.
        /// </summary>
        /// <param name="commandKey">The key of the command.</param>
        /// <value>true or false</value>
        public bool IsCommandChecked(string commandIdentifier)
        {
            if (commandIdentifier == null)
            {
                throw new ArgumentNullException("commandIdentifier");
            }

            Command command = null;

            if (CommandItems.TryGetValue(commandIdentifier, out command))
            {
                return command.Checked;
            }

            return false;
        }

        /// <summary>
        /// Executes the passed command.
        /// </summary>
        /// <remarks>
        /// The command is executed just like the user used the user interface.
        /// </remarks>
        /// <param name="commandKey">The key of the command to execute.</param>
        public void ExecuteCommand(string commandIdentifier)
        {
            if (commandIdentifier == null)
            {
                throw new ArgumentNullException("commandIdentifier");
            }

            Command command = null;

            if (CommandItems.TryGetValue(commandIdentifier, out command))
            {
                this.OnCommandExecuted(command);
            }
        }

        /// <summary>
        /// Hides (makes not visible) the item(s) associated with the passed command key.
        /// </summary>
        /// <param name="commandKey">The key of the command to hide.</param>
        public void HideCommand(String commandIdentifier)
        {
            Command command = null;

            if (!CommandItems.TryGetValue(commandIdentifier, out command))
            {
                throw new ArgumentException();
            }

            if (command.Visible)
            {
                command.Visible = false;
                OnCommandVisibleChanged(command);
            }
        }

        /// <summary>
        /// Shows (makes visible) the item(s) associated with the passed command key.
        /// </summary>
        /// <param name="commandKey">The key of the command to show.</param>
        public void ShowCommand(string commandIdentifier)
        {
            Command command = null;

            if (!CommandItems.TryGetValue(commandIdentifier, out command))
            {
                throw new ArgumentException();
            }

            if (!command.Visible)
            {
                command.Visible = true;
                OnCommandVisibleChanged(command);
            }
        }

        /// <summary>
        /// Enables the item(s) associated with the passed command key.
        /// </summary>
        /// <param name="commandKey">The key of the command to enable.</param>
        public void EnableCommand(string commandIdentifier)
        {
            Command command = null;

            if (!CommandItems.TryGetValue(commandIdentifier, out command))
            {
                throw new ArgumentException();
            }

            if (!command.Enable)
            {
                command.Enable = true;
                OnCommandEnabledChanged(command);
            }
        }

        /// <summary>
        /// Disables the item(s) associated with the passed command key.
        /// </summary>
        /// <param name="commandKey">The key of the command to disable.</param>
        public void DisableCommand(string commandIdentifier)
        {
            Command command = null;

            if (!CommandItems.TryGetValue(commandIdentifier, out command))
            {
                throw new ArgumentException();
            }

            if (command.Enable)
            {
                command.Enable = false;
                OnCommandEnabledChanged(command);
            }
        }

        /// <summary>
        /// Checks the item(s) associated with the passed command key.
        /// </summary>
        /// <remarks>
        /// Check may not be allowed on all items. 
        /// </remarks>
        /// <param name="commandKey">The key of the command to check.</param>
        public void CheckCommand(string commandIdentifier)
        {
            Command command = null;

            if (!CommandItems.TryGetValue(commandIdentifier, out command))
            {
                throw new ArgumentException();
            }

            if (!command.Checked)
            {
                command.Checked = true;
                OnCommandCheckedChanged(command);
            }
        }

        /// <summary>
        /// Unchecks the item(s) associated with the passed command key.
        /// </summary>
        /// <remarks>
        /// Check may not be allowed on all items. 
        /// </remarks>
        /// <param name="commandKey">The key of the command to uncheck.</param>
        public void UncheckCommand(string commandIdentifier)
        {
            Command command = null;

            if (!CommandItems.TryGetValue(commandIdentifier, out command))
            {
                throw new ArgumentException();
            }

            if (command.Checked)
            {
                command.Checked = false;
                OnCommandCheckedChanged(command);
            }
        }

        /// <summary>
        /// Changes the text of the item(s) associated with the passed command key.
        /// </summary>
        /// <param name="commandKey">The key of the command to change text on.</param>
        /// <param name="newText">The new text.</param>
        public void ChangeCommandText(string commandIdentifier, string newText)
        {
            Command command = null;

            if (!CommandItems.TryGetValue(commandIdentifier, out command))
            {
                throw new ArgumentException();
            }

            if (command.Text != newText)
            {
                command.Text = newText;
                OnCommandTextChanged(command);
            }
        }

        /// <summary>
        /// Changes the large image key of the item(s) associated with the passed command key.
        /// </summary>
        /// <param name="commandKey">The key of the command to change the image on.</param>
        /// <param name="newImage">The new image.</param>
        public void ChangeCommandLargeImageKey(string commandIdentifier, String newImageKey)
        {
            Command command = null;

            if (!CommandItems.TryGetValue(commandIdentifier, out command))
            {
                throw new ArgumentException();
            }

            if (command.LargeImageKey != newImageKey)
            {
                command.LargeImageKey = newImageKey;
                OnCommandLargeImageKeyChanged(command);
            }
        }

        /// <summary>
        /// Changes the small image key of the item(s) associated with the passed command key.
        /// </summary>
        /// <param name="commandKey">The key of the command to change the image on.</param>
        /// <param name="newImage">The new image.</param>
        public void ChangeCommandSmallImageKey(string commandIdentifier, String newImageKey)
        {
            Command command = null;

            if (!CommandItems.TryGetValue(commandIdentifier, out command))
            {
                throw new ArgumentException();
            }

            if (command.SmallImageKey != newImageKey)
            {
                command.SmallImageKey = newImageKey;
                OnCommandSmallImageKeyChanged(command);
            }
        }

        public void ChangeCommandToolStripItemDisplayStyle(string commandIdentifier, ToolStripItemDisplayStyle newStyle)
        {
            Command command = null;

            if (!CommandItems.TryGetValue(commandIdentifier, out command))
            {
                throw new ArgumentException();
            }

            if (command.ToolStripItemDisplayStyle != newStyle)
            {
                command.ToolStripItemDisplayStyle = newStyle;
                OnCommandToolStripItemDisplayStleChanged(command);
            }
        }

        #endregion

        #region Event Raiser

        private void OnLargeImageListChanged()
        {
            EventHandler handler = LargeImageListChanged;

            if (handler != null)
            {
                handler(this, EventArgs.Empty);
            }
        }

        private void OnSmallImageListChanged()
        {
            EventHandler handler = SmallImageListChanged;

            if (handler != null)
            {
                handler(this, EventArgs.Empty);
            }
        }

        private void OnCommandExecuted(Command commandItem)
        {
            EventHandler<CommandManagerEventArgs> handler = CommandExecuted;

            if (handler != null)
            {
                handler(this, new CommandManagerEventArgs(commandItem));
            }
        }

        private void OnCommandEnabledChanged(Command commandItem)
        {
            EventHandler<CommandManagerEventArgs> handler = CommandEnabledChanged;

            if (handler != null)
            {
                handler(this, new CommandManagerEventArgs(commandItem));
            }
        }

        private void OnCommandVisibleChanged(Command commandItem)
        {
            EventHandler<CommandManagerEventArgs> handler = CommandVisibleChanged;

            if (handler != null)
            {
                handler(this, new CommandManagerEventArgs(commandItem));
            }
        }

        private void OnCommandCheckedChanged(Command commandItem)
        {
            EventHandler<CommandManagerEventArgs> handler = CommandCheckedChanged;

            if (handler != null)
            {
                handler(this, new CommandManagerEventArgs(commandItem));
            }
        }

        private void OnCommandTextChanged(Command commandItem)
        {
            EventHandler<CommandManagerEventArgs> handler = CommandTextChanged;

            if (handler != null)
            {
                handler(this, new CommandManagerEventArgs(commandItem));
            }
        }

        private void OnCommandToolStripItemDisplayStleChanged(Command command)
        {
            EventHandler<CommandManagerEventArgs> handler = CommandToolStripItemDisplayStleChanged;

            if (handler != null)
            {
                handler(this, new CommandManagerEventArgs(command));
            }
        }

        private void OnCommandLargeImageKeyChanged(Command commandItem)
        {
            EventHandler<CommandManagerEventArgs> handler = CommandLargeImageKeyChanged;

            if (handler != null)
            {
                handler(this, new CommandManagerEventArgs(commandItem));
            }
        }

        private void OnCommandSmallImageKeyChanged(Command commandItem)
        {
            EventHandler<CommandManagerEventArgs> handler = CommandSmallImageKeyChanged;

            if (handler != null)
            {
                handler(this, new CommandManagerEventArgs(commandItem));
            }
        }

        #endregion
    }

}
