using System;
using System.Collections.Generic;
using System.Text;
using System.Reflection;
using System.IO;
using System.Collections.Specialized;
using System.Collections;

namespace LineCounter.Application
{
    internal class CounterManager
    {
        private static CounterManager _current;
        private Dictionary<String, CounterItem> _counters;
        private Dictionary<String, CounterItemCollection> _countersByExtension;
        private Dictionary<String, CounterItem> _preferredCounter;


        /// <summary>
        /// Initializes a new instance of the <see cref="T:CounterManager"/> class.
        /// </summary>
        private CounterManager()
        {
            _counters = new Dictionary<String, CounterItem>();
            _countersByExtension = new Dictionary<string, CounterItemCollection>();
            _preferredCounter = new Dictionary<String, CounterItem>();
        }


        public static CounterManager Current
        {
            get
            {
                if (_current == null)
                {
                    _current = new CounterManager();
                }

                return _current;
            }
        }

        private Dictionary<String, CounterItem> Counters
        {
            get { return _counters; }
        }

        private Dictionary<String, CounterItemCollection> CountersByExtension
        {
            get { return _countersByExtension; }
        }

        public ICollection<CounterItem> CounterItems
        {
            get { return Counters.Values; }
        }

        public ICollection<String> SupportedExtensions
        {
            get { return CountersByExtension.Keys; }
        }

        private Dictionary<String, CounterItem> PreferredCounter
        {
            get { return _preferredCounter; }
        }


        /// <summary>
        /// Clears user's preferred counter of this extension.
        /// </summary>
        /// <exception cref="ArgumentNullException"/>
        public void ClearPreferredCounter(String extension)
        {
            PreferredCounter.Remove(extension);
        }

        /// <summary>
        /// Gets all avaliable counters' name.
        /// </summary>
        /// <returns></returns>
        public String[] GetCounterNames()
        {
            String[] result = new String[Counters.Count];
            int i = 0;

            foreach(CounterItem counterItem in Counters.Values)
            {
                result[i++] = counterItem.Id;
            }

            return result;
        }

        public CounterItem GetCounter(string name)
        {
            CounterItem result = null;
            Counters.TryGetValue(name, out result);

            return result;
        }

        /// <summary>
        /// Get counters associated with the extension.
        /// </summary>
        /// <exception cref="ArgumentNullException" />
        /// <exception cref="CounterNotFoundException" />
        public CounterItemCollection GetCountersByExtension(String extension)
        {
            if (extension == null)
            {
                throw new ArgumentNullException("extension");
            }

            CounterItemCollection result = null;
            CountersByExtension.TryGetValue(extension, out result);

            if (result == null)
            {
                throw new CounterNotFoundException(extension);
            }

            return result;
        }

        public CounterItem GetCounterByFile(String file, Boolean forceChoosingCounter)
        {
            String extension = FileHelper.GetExtension(file);
            CounterItem counter = null;

            // if we don't have a counter can handle this kind of
            // file, we'll use a general counter instead.
            if (!this.CountersByExtension.ContainsKey(extension))
            {
                extension = ".*";
            }

            if (!forceChoosingCounter)
            {
                PreferredCounter.TryGetValue(extension, out counter);
            }

            if (counter == null)
            {

                CounterItemCollection counters = GetCountersByExtension(extension);

                if (counters.Count == 1)
                {
                    IEnumerator enumerator = counters.GetEnumerator();
                    enumerator.MoveNext();
                    return (CounterItem)enumerator.Current;
                }
                else
                {
                    CounterChooserResult result = ApplicationManager.Current.MainForm.PromptForChoosingCounter(counters, file, extension);

                    counter = result.SelectedCounterItem;

                    if (result.ApplyToTheSameExtension)
                    {
                        PreferredCounter.Add(extension, result.SelectedCounterItem);
                    }
                }
            }

            return counter;
        }

        public void LoadCounters(string countersFolder)
        {
            Counters.Clear();
            CountersByExtension.Clear();
            List<CounterItem> counterItems;

            if (!Directory.Exists(countersFolder))
            {
                Directory.CreateDirectory(countersFolder);
            }
            string[] assemblyFiles = Directory.GetFiles(countersFolder, "*.dll", SearchOption.AllDirectories); 

            foreach (string assemblyFile in assemblyFiles)
            {
                Assembly assembly = Assembly.LoadFile(assemblyFile);

                if (TryLoadCounterFromAssembly(assembly, out counterItems))
                {
                    counterItems.ForEach(new Action<CounterItem>(
                        delegate(CounterItem item) 
                        {
                            AppendAcceptableExtensions(item.GetAcceptableExtensions(), item);
                            Counters.Add(item.Id, item); 
                        }));
                }
            }
        }

        private void AppendAcceptableExtensions(string[] extensions, CounterItem item)
        {
            foreach (String extension in extensions)
            {
                if (CountersByExtension.ContainsKey(extension))
                {
                    CountersByExtension[extension].Add(item);
                }
                else
                {
                    CounterItemCollection collection = new CounterItemCollection();
                    collection.Add(item);
                    CountersByExtension.Add(extension, collection);
                }
            }
        }

        private static bool TryLoadCounterFromAssembly(Assembly assembly, out List<CounterItem> counterItems)
        {
            counterItems = new List<CounterItem>();
            Counter counter = null;

            foreach (Type type in assembly.GetTypes())
            {
                if (type.IsSubclassOf(typeof(Counter)))
                {
                    CounterOptionsDescriptorBase optionsDescriptor = null;

                    // Counter class found, now try to find CounterAttribute
                    Object[] foundCounterAttribute = type.GetCustomAttributes(typeof(CounterAttribute), false);

                    if (foundCounterAttribute != null)
                    {
                        // The attribute found, let's instance the PropertyDescriptor and get the counter's information
                        try
                        {
                            CounterAttribute counterAttribute = (CounterAttribute)foundCounterAttribute[0];

                            if (counterAttribute.HasEditableOptions)
                            {
                                optionsDescriptor = (CounterOptionsDescriptorBase)counterAttribute.OptionsDescriptor.GetConstructor(new Type[0]).Invoke(null);
                            }

                            try
                            {
                                // The PropertyDescriptor initialize succeed, now create a new Counter instance
                                counter = (Counter)type.GetConstructor(new Type[0]).Invoke(null);

                                if (counterAttribute.HasEditableOptions)
                                {
                                    counter.Options = optionsDescriptor.GetOptions();
                                }

                                CounterItem counterItem = new CounterItem(counterAttribute.Name, counterAttribute.Id, counter, optionsDescriptor, counterAttribute.AcceptableExtensions);
                                counterItem.Description = counterAttribute.Description;

                                // Everything's Ok now, go back to the caller
                                counterItems.Add(counterItem);
                            }
                            catch(Exception ex)
                            {
                                ApplicationDiagnostics.LogException(ex);

                                // Instancing Counter failed, continue the foreach loop
                                continue;
                            }
                        }
                        catch(Exception ex)
                        {
                            ApplicationDiagnostics.LogException(ex);

                            // Instancing PropertyDescriptor failed, continue the foreach loop
                            continue;
                        }
                    }
                }
            }

            // No Counter class found, return with false value
            return counterItems.Count > 0;
        }
    }
}
