using System;
using System.Collections.Generic;
using System.Text;
using LineCounter.Files;

namespace LineCounter.Application
{

    /// <summary>
    /// Represents the manager for the operations of all files and folders created in the application.
    /// </summary>
    public class FileManager
    {
        #region Events

        public event EventHandler<CommonFileActionEventArgs> FileAdded;
        public event EventHandler<FileMoveEventArgs> FileMoved;
        public event EventHandler<FileMoveEventArgs> FileRemoved;

        #endregion


        #region Local Variables

        private RootFileContainer _root;

        #endregion


        #region Constractor

        /// <summary>
        /// Initializes a new instance of the <see cref="T:FileManager"/> class.
        /// </summary>
        public FileManager() { }

        public void Initialize()
        {
            _root = new RootFileContainer();
            OnFileAdded(_root);
        }

        public void Reset()
        {
            if (_root != null)
            {
                OnFileRemoved(_root, null);
            }

            Initialize();
        }

        #endregion


        #region Properties

        /// <summary>
        /// Represents the root of the file tree.
        /// </summary>
        /// <remarks>This property is read only.</remarks>
        public RootFileContainer Root
        {
            get { return _root; }
        }

        #endregion


        #region Public Operations

        /// <summary>
        /// Gets file container by name.
        /// </summary>
        /// <returns>The file container. Null value will be returned if there's no such container.</returns>
        /// <exception cref="ArgumentNullException" />
        /// <exception cref="FileContainerNotFoundException" />
        public IFileContainer GetContainer(String name)
        {
            if (StringHelper.IsNullOrEmpty(name))
            {
                throw new ArgumentNullException("name");
            }

            IFileContainer container = Root;

            if (name == container.FullName)
            {
                // Requesting the root container
                return container;
            }

            if (name.IndexOf(container.FullName) == -1)
            {
                throw new FileContainerNotFoundException(name, FileHelper.GetFileName(name));
            }
            else
            {
                name = name.Substring(container.FullName.Length + 1);
            }
            

            foreach (String subName in name.Split(new Char[] { System.IO.Path.DirectorySeparatorChar }, StringSplitOptions.None))
            {
                if (container != null)
                {
                    container.SubContainers.TryGetValue(subName, out container);
                }
                else
                {
                    throw new FileContainerNotFoundException(name, FileHelper.GetFileName(name));
                }
            }

            return container;
        }

        /// <summary>
        /// Gets file by name.
        /// </summary>
        /// <param name="name">The file name.</param>
        /// <param name="containerName">Name of the file's container.</param>
        /// <returns>The file. Null value will be returned if there's no such file.</returns>
        public IFile GetFile(String name, String containerName)
        {
            if (StringHelper.IsNullOrEmpty(name))
            {
                throw new ArgumentNullException("name");
            }

            if (StringHelper.IsNullOrEmpty(containerName))
            {
                throw new ArgumentNullException("containerName");
            }

            IFileContainer container = GetContainer(containerName);

            if (container == null)
            {
                throw new FileContainerNotFoundException(containerName, FileHelper.GetFileName(containerName));
            }

            IFile file = null;

            container.Files.TryGetValue(name, out file);

            return file;
        }

        public IFile GetFile(String fullName)
        {
            if (StringHelper.IsNullOrEmpty(fullName))
            {
                throw new ArgumentNullException("fullName");
            }

            int index = fullName.LastIndexOf(System.IO.Path.DirectorySeparatorChar);
            if (index < 0)
            {
                throw new FileNotFoundException(fullName, FileHelper.GetFileName(fullName));
            }

            String containerName = fullName.Remove(index);

            return GetFile(FileHelper.GetFileName(fullName), containerName);
        }


        public FileDictionary GetFiles(String containerName)
        {
            if (StringHelper.IsNullOrEmpty(containerName))
            {
                throw new ArgumentNullException("containerName");
            }

            IFileContainer container = GetContainer(containerName);

            if (container != null)
            {
                return container.Files;
            }

            return null;
        }

        /// <summary>
        /// Adds the file to a specific container.
        /// </summary>
        /// <param name="file">The file.</param>
        /// <param name="containerName">Name of the container.</param>
        /// <exception cref="FileExistsException" />
        /// <exception cref="FileContainerNotFoundException" />
        public void AddFile(IFile file, String containerName)
        {
            if (file == null)
            {
                throw new ArgumentNullException("file");
            }

            if (StringHelper.IsNullOrEmpty(containerName))
            {
                throw new ArgumentNullException("containerName");
            }

            IFileContainer container = GetContainer(containerName);

            if (container != null)
            {
                container.AddFile(file);
            }
            else
            {
                throw new FileContainerNotFoundException(containerName, FileHelper.GetFileName(containerName));
            }

            OnFileAdded(file);
        }

        /// <summary>
        /// Adds the container to a specific container.
        /// </summary>
        /// <param name="container">The container.</param>
        /// <param name="parentContainerName">Name of the parent container.</param>
        /// <exception cref="FileContainerNotFoundException" />
        /// <exception cref="FileContainerExistsException" />
        public void AddContainer(IFileContainer container, String parentContainerName)
        {
            if (container == null)
            {
                throw new ArgumentNullException("container");
            }

            if (StringHelper.IsNullOrEmpty(parentContainerName))
            {
                throw new ArgumentNullException("parentContainerName");
            }

            parentContainerName = StringHelper.MaskNullString(parentContainerName);

            if (parentContainerName.Length == 0)
            {
                parentContainerName = RootFileContainer.RootContainerName;
            }

            IFileContainer parentContainer = GetContainer(parentContainerName);

            if (parentContainer != null)
            {
                parentContainer.AddFileContainer(container);
            }
            else
            {
                throw new FileContainerNotFoundException(parentContainerName, FileHelper.GetFileName(parentContainerName));
            }

            OnFileAdded(container);
        }

        /// <summary>
        /// Moves the internal file or file container to the target container.
        /// </summary>
        /// <param name="file">The file or file container to be moved.</param>
        /// <remarks>
        /// Use the <code>MoveContainer</code> method instead if you know the
        /// file object is of the <typeparamref name="IFileContainer"/> type.
        /// </remarks>
        public void MoveFile(IFile file, IFileContainer targetContainer)
        {
            if (file == null)
            {
                throw new ArgumentNullException("file");
            }

            if (targetContainer == null)
            {
                throw new ArgumentNullException("targetContainer");
            }

            if (file.Type == FileType.File)
            {
                if (file.Container != targetContainer)
                {
                    IFileContainer origionalContainer = file.Container;

                    if (file.Container != null)
                    {
                        file.Container.RemoveFile(file);
                    }

                    targetContainer.AddFile(file);

                    OnFileMoved(file, origionalContainer);
                }
            }
            else
            {
                MoveContainer((IFileContainer)file, targetContainer);
            }
        }

        /// <summary>
        /// Moves a file container to the target container.
        /// </summary>
        /// <exception cref="FileOperationNotSupportedException"/>
        public void MoveContainer(IFileContainer container, IFileContainer targetContainer)
        {
            if (container == null)
            {
                throw new ArgumentNullException("container");
            }

            if (targetContainer == null)
            {
                throw new ArgumentNullException("targetContainer");
            }

            if (container.Contains(targetContainer))
            {
                throw new FileOperationNotSupportedException(container.FullName, container.FullNameDisplayText, FileOperation.Move);
            }

            if (container.Container != targetContainer)
            {
                IFileContainer origionalContainer = container.Container;

                if (container.Container != null)
                {
                    container.Container.RemoveFileContainer(container);
                }

                targetContainer.AddFileContainer(container);

                OnFileMoved(container, origionalContainer);
            }
        }

        public void MoveContainer(IFileContainer container, String targetContainerName)
        {
            if (container == null)
            {
                throw new ArgumentNullException("container");
            }

            if (StringHelper.IsNullOrEmpty(targetContainerName))
            {
                throw new ArgumentNullException("targetContainerName");
            }

            IFileContainer targetContainer = GetContainer(targetContainerName);

            if (targetContainer != null)
            {
                MoveContainer(container, targetContainer);
            }
            else
            {
                throw new FileContainerNotFoundException(targetContainerName, FileHelper.GetFileName(targetContainerName));
            }
        }

        /// <summary>
        /// Removes the file from collection.
        /// </summary>
        /// <exception cref="ArgumentNullException"/>
        /// <exception cref="FileContainerNotFoundException" />
        /// <exception cref="FileNotFoundException" />
        public void RemoveFile(IFile file)
        {
            if (file == null)
            {
                throw new ArgumentNullException("file");
            }

            IFileContainer container = file.Container;

            if (container == null)
            {
                throw new FileContainerNotFoundException(container.FullName, container.FullNameDisplayText);
            }

            if (!container.Files.ContainsKey(file.Name))
            {
                throw new FileNotFoundException(file.FullName, file.FullNameDisplayText);
            }

            container.Files.Remove(file.Name);

            OnFileRemoved(file, container);
        }

        /// <summary>
        /// Removes the file container from collection.
        /// </summary>
        /// <exception cref="ArgumentNullException"/>
        /// <exception cref="FileContainerNotFoundException" />
        public void RemoveFileContainer(IFileContainer container)
        {
            if (container == null)
            {
                throw new ArgumentNullException("container");
            }

            if (container.ContainerType == FileContainerType.Root)
            {
                throw new FileContainerTypeNotAcceptableException(container.FullName, container.DisplayName);
            }

            IFileContainer parentContainer = container.Container;

            if (parentContainer == null)
            {
                throw new FileContainerNotFoundException(parentContainer.FullName, parentContainer.FullNameDisplayText);
            }

            if (!parentContainer.SubContainers.ContainsKey(container.Name))
            {
                throw new FileContainerNotFoundException(container.FullName, container.FullNameDisplayText);
            }

            parentContainer.SubContainers.Remove(container.Name);

            OnFileRemoved(container, parentContainer);
        }

        #endregion


        #region Event Firer

        private void OnFileAdded(IFile file)
        {
            if (FileAdded != null)
            {
                FileAdded(this, new CommonFileActionEventArgs(file, FileAction.New));
            }
        }

        private void OnFileMoved(IFile file, IFileContainer origionalContainer)
        {
            if (FileMoved != null)
            {
                FileMoved(this, new FileMoveEventArgs(origionalContainer, file, FileAction.Move));
            }
        }

        private void OnFileRemoved(IFile file, IFileContainer origionalContainer)
        {
            if (FileRemoved != null)
            {
                FileRemoved(this, new FileMoveEventArgs(origionalContainer, file, FileAction.Remove));
            }
        }

        #endregion
    }
}
