using System;
using System.Collections.Generic;
using System.Text;
using LineCounter.Files;

namespace LineCounter.Application
{
    public class FileModel
    {
        #region Event Declearations

        /// <summary>
        /// Occours when a file or an object which type inherited from IFile (such as IFileContainer)
        /// is actived by user.
        /// </summary>
        public event EventHandler<AfterFileActiveEventArgs> FileActiveded;

        /// <summary>
        /// Occours when information (count result, address, etc.) of
        /// a file or an object which type inherited from IFile (such as IFileContainer)
        /// changed.
        /// </summary>
        public event EventHandler<CommonFileActionEventArgs> FileInformationChanged;

        #endregion

        #region Local Varaiables

        private static FileModel _instance;
        private FileManager _fileManager;
        private IFileContainer _activeFileContainer;

        #endregion

        #region Constractor

        public FileModel()
        {
            _fileManager = new FileManager();
            _fileManager.FileMoved += new EventHandler<FileMoveEventArgs>(OnFileMoved);
            _fileManager.FileRemoved += new EventHandler<FileMoveEventArgs>(OnFileMoved);
        }

        #endregion

        #region Properties

        public static FileModel Instance
        {
            get
            {
                if (_instance == null)
                {
                    _instance = new FileModel();
                }

                return _instance;
            }
        }

        public FileManager FileManager
        {
            get { return _fileManager; }
        }

        public IFileContainer ActiveFileContainer
        {
            get { return _activeFileContainer; }
            private set { _activeFileContainer = value; }
        }

        #endregion

        #region Public Operations

        public void Initialize()
        {
            _fileManager.Initialize();

            ActiveFile(_fileManager.Root);
        }

        /// <summary>
        /// Active a file.
        /// </summary>
        /// <exception cref="ArgumentNullException" />
        public void ActiveFile(IFile file)
        {
            if (file == null)
            {
                throw new ArgumentNullException("file");
            }

            if (file.Type == FileType.FileContainer)
            {
                _activeFileContainer = (IFileContainer)file;
            }

            OnFileActiveded(file);
        }

        public void CountFile(Boolean forceChoosingCounter, params String[] fileNames)
        {
            if (fileNames == null)
            {
                throw new ArgumentNullException("fileNames");   
            }

            foreach (String fileName in fileNames)
            {
                IFile file = FileManager.GetFile(fileName);

                CountFile(forceChoosingCounter, file);
            }
        }

        public void CountFilesInContainer(Boolean forceChoosingCounter, String containerName, Boolean topContainerOnly)
        {
            IFileContainer container = FileManager.GetContainer(containerName);

            CountFileContainer(forceChoosingCounter, container, topContainerOnly);
        }

        private void CountFile(Boolean forceChoosingCounter, IFile file)
        {
            bool count = false;
            switch (file.ContentType)
            {
                case FileContentType.Countable:
                    count = true;
                    break;
                case FileContentType.Ignored:
                    break;
                case FileContentType.Unknown:
                    count = true;
                    forceChoosingCounter = true;
                    break;
            }

            if (count)
            {
                try
                {
                    CounterItem counter = CounterManager.Current.GetCounterByFile(
                        file.Address, forceChoosingCounter);

                    file.CounterResult = counter.Counter.Count(file);
                    file.Counted = true;

                    OnFileInformationChanged(file);
                }
                catch (CounterNotFoundException counterNotFoundEx)
                {
                    ApplicationDiagnostics.LogException(counterNotFoundEx);

                    throw;
                }
            }
        }

        private void CountFileContainer(Boolean forceChoosingCounter, IFileContainer container, Boolean topContainerOnly)
        {
            foreach (IFile file in container.GetFilesByType(FileContentType.Countable, topContainerOnly))
            {
                CountFile(forceChoosingCounter, file);
            }

            container.Counted = true;

            OnFileInformationChanged(container);
        }

        /// <summary>
        /// Imports file into the application.
        /// </summary>
        /// <exception cref="FileExistsException" />
        /// <exception cref="FileContainerNotFoundException" />
        /// <exception cref="FileContainerExistsException" />
        public void ImportFile(String[] fileNames, String containerName)
        {
            if (fileNames != null)
            {
                foreach (String fileName in fileNames)
                {
                    IFile file = FileImportingHelper.GetFileObject(fileName);

                    if (file != null)
                    {
                        switch (file.Type)
                        {
                            case FileType.File:
                                this.FileManager.AddFile(file, containerName);
                                break;
                            case FileType.FileContainer:
                                this.FileManager.AddContainer((IFileContainer)file, containerName);
                                break;
                        }
                    }
                }
            }
        }

        public static void LunchFile(IFile file)
        {
            FileLauncher.Lunch(file);
        }

        #endregion

        #region Event Hander

        private void OnFileMoved(object sender, FileMoveEventArgs e)
        {
            if (e.File == ActiveFileContainer || 
                (e.File.Type == FileType.FileContainer &&
                ((IFileContainer)e.File).ContainerType == FileContainerType.Root))
            {
                ActiveFileContainer = null;
            }
        }

        #endregion

        #region Event Raiser

        private void OnFileActiveded(IFile file)
        {
            if (FileActiveded != null)
            {
                FileActiveded(this, new AfterFileActiveEventArgs(file));
            }
        }

        private void OnFileInformationChanged(IFile file)
        {
            if (FileInformationChanged != null)
            {
                FileInformationChanged(this, new CommonFileActionEventArgs(file, FileAction.InformationChanged));
            }
        }

        #endregion
    }
}
