using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using LineCounter.Application;

namespace LineCounter.Application.UI
{
    internal partial class ApplicationMainForm : Form
    {
        private OptionsForm _optionsForm;
        private ToolStripMenuItem checkedFileListFilterItem;
        private Dictionary<String, ToolStripMenuItem> fileListFilterStripMenuItems;


        #region Initializers

        internal ApplicationMainForm()
        {
            InitializeComponent();

            fileImporter.ExtensionFilter = CounterManager.Current.SupportedExtensions;

            InitializeCommandBar();

            BindEvents();
        }


        private void InitializeCommandBar()
        {
            // Initlize file list filter
            fileListFilterStripMenuItems = new Dictionary<string, ToolStripMenuItem>();

            ToolStripMenuItem item;

            // Append "All" filter type to the list
            item = new ToolStripMenuItem(Resources.Resource.GetLocalizedString("Text_All"));
            item.Tag = null;
            item.Click += new EventHandler(fileListFilterMenuItem_Click);
            item.Checked = !ApplicationRuntimeSettings.EnableFileContentFilter;
            checkedFileListFilterItem = item;
            toolStripSplitButton_FileFilter.DropDown.Items.Add(item);
            fileListFilterStripMenuItems.Add("All", item);

            Array fileTypes = Enum.GetValues(typeof(FileContentType));

            // Append each filter type to the list
            foreach (FileContentType fileTypeInEnum in fileTypes)
            {
                item = new ToolStripMenuItem(Resources.Resource.GetLocalizedString("Text_" + fileTypeInEnum.ToString()));
                item.Tag = new Nullable<FileContentType>(fileTypeInEnum);
                item.Click += new EventHandler(fileListFilterMenuItem_Click);
                toolStripSplitButton_FileFilter.DropDown.Items.Add(item);
                fileListFilterStripMenuItems.Add(fileTypeInEnum.ToString(), item);

                if (ApplicationRuntimeSettings.EnableFileContentFilter && ApplicationRuntimeSettings.FileContentFilter == fileTypeInEnum)
                {
                    item.Checked = true;
                }
            }

            // Initlize menu and tool strip buttons status via settings
            toolStripButton_ShowFileIcons.Checked = ApplicationRuntimeSettings.ShowFileIcon;
            toolStripButton_ShowFilesInSubfolders.Checked = ApplicationRuntimeSettings.ShowAllFilesInSubfolder;
        }

        public void BindEvents()
        {
            ApplicationRuntimeSettings.FileContentFilterChanged += new EventHandler(ApplicationRuntimeSettings_FileContentFilterChanged);
            ApplicationRuntimeSettings.ShowAllFilesInSubfolderChanged += new EventHandler(ApplicationRuntimeSettings_ShowAllFilesInSubfolderChanged);
            ApplicationRuntimeSettings.ShowFileIconChanged += new EventHandler(ApplicationRuntimeSettings_ShowFileIconChanged);        }

        #endregion

        #region Application Event Handler

        private void ApplicationRuntimeSettings_ShowFileIconChanged(object sender, EventArgs e)
        {
            toolStripButton_ShowFileIcons.Checked = ApplicationRuntimeSettings.ShowFileIcon;
        }

        private void ApplicationRuntimeSettings_ShowAllFilesInSubfolderChanged(object sender, EventArgs e)
        {
            toolStripButton_ShowFilesInSubfolders.Checked = ApplicationRuntimeSettings.ShowAllFilesInSubfolder;
        }

        private void ApplicationRuntimeSettings_FileContentFilterChanged(object sender, EventArgs e)
        {
            if (checkedFileListFilterItem != null)
            {
                checkedFileListFilterItem.Checked = false;
            }

            if (ApplicationRuntimeSettings.EnableFileContentFilter)
            {
                checkedFileListFilterItem = fileListFilterStripMenuItems[ApplicationRuntimeSettings.FileContentFilter.Value.ToString()];
            }
            else
            {
                checkedFileListFilterItem = fileListFilterStripMenuItems["All"];
            }

            checkedFileListFilterItem.Checked = true;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets the options form.
        /// </summary>
        private OptionsForm OptionsForm
        {
            get
            {
                if (_optionsForm == null)
                {
                    _optionsForm = new OptionsForm();
                }

                return _optionsForm;
            }
        }

        #endregion

        #region Operations

        public CounterChooserResult PromptForChoosingCounter(ICollection<CounterItem> counters, String fileName, String extension)
        {
            ChooseCounter chooser = new ChooseCounter(counters, fileName, extension);

            if (chooser.ShowDialog(this) == DialogResult.OK)
            {
                return new CounterChooserResult(chooser.ApplyToSameExtension, chooser.SelectedCounter);
            }
            else
            {
                return new CounterChooserResult(false, null);
            }
        }

        public void ShowAboutBox()
        {
            AboutBox about = new AboutBox();
            about.ShowDialog(this);

            about.Dispose();
        }

        public void ShowFileInformation(IFile file)
        {
            FileInformation form = new FileInformation();
            form.StartPosition = FormStartPosition.CenterScreen;

            form.Show(file, this);
        }

        private void ShowOptionsForm()
        {
            OptionsForm.ShowDialog(this);
        }

        #endregion

        #region Event Handlers


        private void optionsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ShowOptionsForm();
        }

        private void button_Recount_Click(object sender, EventArgs e)
        {
            //Recount();
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ShowAboutBox();
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ApplicationManager.Current.Exit();
        }

        #endregion

        private void toolStripButton_Refresh_Click(object sender, EventArgs e)
        {
            FileModel.Instance.ActiveFile(FileModel.Instance.ActiveFileContainer);
        }

        private void contextMenuStrip_FileList_Opening(object sender, CancelEventArgs e)
        {
            //if (listView_FileList.SelectedItems.Count == 1)
            //{
            //    lunchFileToolStripMenuItem.Enabled = true;
            //}
            //else
            //{
            //    lunchFileToolStripMenuItem.Enabled = false;
            //}
        }

        private void lunchFileToolStripMenuItem_Click(object sender, EventArgs e)
        {
            //IFile file = ((FileListItem)listView_FileList.SelectedItems[0]).File;

            //LunchFile(file);
        }

        #region Command Bar Event Handlers

        private void fileListFilterMenuItem_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem item = (ToolStripMenuItem)sender;

            if (!item.Checked)
            {
                ApplicationRuntimeSettings.FileContentFilter = (Nullable<FileContentType>)item.Tag;
            }
        }

        private void toolStripButton_ImportFiles_Click(object sender, EventArgs e)
        {
            fileImporter.RequestImportFile(FileModel.Instance.ActiveFileContainer.FullName);
        }

        private void toolStripButton_ImportFolder_Click(object sender, EventArgs e)
        {
            fileImporter.RequestImportFolder(FileModel.Instance.ActiveFileContainer.FullName);
        }

        private void toolStripButton_ShowFilesInSubfolders_Click(object sender, EventArgs e)
        {
            ApplicationRuntimeSettings.ShowAllFilesInSubfolder = !ApplicationRuntimeSettings.ShowAllFilesInSubfolder;
        }

        private void toolStripButton_ShowFileIcons_Click(object sender, EventArgs e)
        {
            ApplicationRuntimeSettings.ShowFileIcon = !ApplicationRuntimeSettings.ShowFileIcon;
        }

        private void toolStripButton_CountSelectedFile_Click(object sender, EventArgs e)
        {
            foreach (String file in fileList.GetSelectedFileNames())
            {
                FileModel.Instance.CountFile(false, file);
            }
        }

        private void OnCountFolderRequested(object sender, EventArgs e)
        {
            //FileModel.Instance.CountFilesInContainer(false, FileModel.Instance.ActiveFileContainer.FullName, !ApplicationRuntimeSettings.ShowAllFilesInSubfolder);
            FileModel.Instance.CountFilesInContainer(false, FileModel.Instance.ActiveFileContainer.FullName, false);
        }

        #endregion

        private void toolStripMenuItem_ImportFolder_Click(object sender, EventArgs e)
        {
            fileImporter.RequestImportFolder(FileModel.Instance.ActiveFileContainer.FullName);
        }

        private void toolStripMenuItem_ImportFiles_Click(object sender, EventArgs e)
        {
            fileImporter.RequestImportFile(FileModel.Instance.ActiveFileContainer.FullName);
        }

        private void ApplicationMainForm_KeyDown(object sender, KeyEventArgs e)
        {
            switch (e.KeyCode)
            {
                case Keys.A:
                    {
                        if (e.Control)
                        {
                            fileList.SelectAllItems();
                        }

                        break;
                    }
            }
        }

        private void toolStripMenuItem_SelectAll_Click(object sender, EventArgs e)
        {
            fileList.SelectAllItems();
        }

        private void toolStripMenuItem_RemoveSelectedFiles_Click(object sender, EventArgs e)
        {
            fileList.RemoveSelectedFiles();
        }

        private void toolStripMenuItem_RemoveAllFiles_Click(object sender, EventArgs e)
        {
            fileList.RemoveAllFiles();
        }

        private void toolStripMenuItem_Reset_Click(object sender, EventArgs e)
        {
            FileModel.Instance.FileManager.Reset();
        }
    }
}