using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using LineCounter.Application;

namespace LineCounter.Application.UI
{
    public partial class FileList : UserControl
    {
        #region Local Varaiables

        private ImageList _imageList;
        private FileImporter _fileImporter;

        #endregion

        public FileList()
        {
            InitializeComponent();

            BindEvent();
        }

        #region Properties

        [Category("Behavior")]
        public ImageList ImageList
        {
            get { return _imageList; }
            set { _imageList = value; }
        }

        [Category("Behavior")]
        public FileImporter FileImporter
        {
            get
            {
                if (_fileImporter == null)
                {
                    _fileImporter = new FileImporter();
                }

                return _fileImporter;
            }
            set { _fileImporter = value; }
        }

        #endregion

        private void BindEvent()
        {
            ApplicationRuntimeSettings.FileContentFilterChanged += new EventHandler(ApplicationRuntimeSettings_FilterFileSettingsChanged);
            ApplicationRuntimeSettings.ShowAllFilesInSubfolderChanged += new EventHandler(ApplicationRuntimeSettings_FilterFileSettingsChanged);
            ApplicationRuntimeSettings.ShowFileIconChanged += new EventHandler(ApplicationRuntimeSettings_ShowFileIconChanged);

            FileModel.Instance.FileInformationChanged += new EventHandler<CommonFileActionEventArgs>(Instance_FileInformationChanged);
            FileModel.Instance.FileActiveded += new EventHandler<AfterFileActiveEventArgs>(Instance_AfterFileActive);
            FileModel.Instance.FileManager.FileAdded += new EventHandler<CommonFileActionEventArgs>(FileManager_FileAdded);
            FileModel.Instance.FileManager.FileMoved += new EventHandler<FileMoveEventArgs>(FileManager_FileMoved);
            FileModel.Instance.FileManager.FileRemoved += new EventHandler<FileMoveEventArgs>(FileManager_FileRemoved);
        }


        private void ApplicationRuntimeSettings_FilterFileSettingsChanged(object sender, EventArgs e)
        {
            ShowFiles(FileModel.Instance.ActiveFileContainer, false);
        }

        private void ApplicationRuntimeSettings_ShowFileIconChanged(object sender, EventArgs e)
        {
            if (ApplicationRuntimeSettings.ShowFileIcon)
            {
                listView_FileList.SmallImageList = ImageList;
            }
            else
            {
                listView_FileList.SmallImageList = null;
            }

            // Stop the ListView updating the newly added items
            listView_FileList.BeginUpdate();

            foreach (FileListItem item in listView_FileList.Items)
            {
                item.ApplyIcon(item.File);
            }

            // Let the ListView update the UI
            listView_FileList.EndUpdate();
        }


        public IEnumerable<String> GetSelectedFileNames()
        {
            foreach (FileListItem item in listView_FileList.SelectedItems)
            {
                yield return item.Name;
            }
        }

        public void RefreshList()
        {
            ShowFiles(FileModel.Instance.ActiveFileContainer, false);
        }


        private void ShowFiles(IFileContainer container, Boolean append)
        {
            if (ApplicationRuntimeSettings.ShowFileIcon)
            {
                listView_FileList.SmallImageList = ImageList;
            }
            else
            {
                listView_FileList.SmallImageList = null;
            }

            if (!append)
            {
                listView_FileList.Items.Clear();
            }

            if (container == null)
            {
                return;
            }

            IList<ListViewItem> items = GetListViewItems(container,
                                                        ApplicationRuntimeSettings.ShowAllFilesInSubfolder,
                                                        ApplicationRuntimeSettings.EnableFileContentFilter,
                                                        ApplicationRuntimeSettings.FileContentFilter);

            // Stop the ListView updating the newly added items
            listView_FileList.BeginUpdate();

            foreach (ListViewItem item in items)
            {
                listView_FileList.Items.Add(item);
            }

            // Let the ListView update the UI
            listView_FileList.EndUpdate();
        }

        private IList<ListViewItem> GetListViewItems(IFileContainer container, Boolean showFilesInSubfolders, Boolean enableContentTypeFilter, Nullable<FileContentType> fileFilterType)
        {
            List<ListViewItem> result = new List<ListViewItem>(); 

            foreach (IFile file in container.Files.Values)
            {
                if (enableContentTypeFilter && file.ContentType != fileFilterType.Value)
                {
                    continue;
                }

                FileListItem item = new FileListItem(file);
                listView_FileList.Items.Add(item);
                item.Name = file.FullName;
                item.ApplyIcon(item.File);
            }

            if (showFilesInSubfolders)
            {
                foreach (IFileContainer subContainer in container.SubContainers.Values)
                {
                    result.AddRange(GetListViewItems(subContainer, true, enableContentTypeFilter, fileFilterType));
                }
            }

            return result;
        }

        private static Boolean ShouldInList(IFile file)
        {
            Boolean result = file.Type == FileType.File &&  // Check file type
                    (file.Container == FileModel.Instance.ActiveFileContainer || (FileModel.Instance.ActiveFileContainer.Contains(file) && ApplicationRuntimeSettings.ShowAllFilesInSubfolder)) &&  // Check whether file is in container
                    (!ApplicationRuntimeSettings.EnableFileContentFilter || file.ContentType == ApplicationRuntimeSettings.FileContentFilter.Value);    // Check if file is filtered

            return result;
        }

        private static void LunchFile(ListViewItem item)
        {
            try
            {
                IFile file = FileModel.Instance.FileManager.GetFile(item.Name);

                FileModel.LunchFile(file);
            }
            catch
            {
                throw;
            }
        }

        public void RemoveSelectedFiles()
        {
            foreach (FileListItem item in listView_FileList.SelectedItems)
            {
                FileModel.Instance.FileManager.RemoveFile(item.File);
            }
        }

        public void RemoveAllFiles()
        {
            foreach (FileListItem item in listView_FileList.Items)
            {
                FileModel.Instance.FileManager.RemoveFile(item.File);
            }
        }

        public void SelectAllItems()
        {
            foreach (ListViewItem item in listView_FileList.Items)
            {
                if (!item.Selected)
                {
                    item.Selected = true;
                }
            }
        }


        private void FileManager_FileRemoved(object sender, FileMoveEventArgs e)
        {
            if (ApplicationRuntimeSettings.ShowAllFilesInSubfolder)
            {
                RefreshList();
            }
            else
            {
                switch (e.File.Type)
                {
                    case FileType.File:
                        {
                            if (ShouldInList(e.File))
                            {
                                listView_FileList.Items[e.File.FullName].Remove();
                            }

                            break;
                        }
                    case FileType.FileContainer:
                        {
                            if (FileModel.Instance.ActiveFileContainer == null)
                            {
                                // The showing files is already been removed.
                                this.listView_FileList.Items.Clear();
                            }

                            break;
                        }
                }
            }
        }

        private void FileManager_FileMoved(object sender, FileMoveEventArgs e)
        {
            if (ApplicationRuntimeSettings.ShowAllFilesInSubfolder)
            {
                RefreshList();
            }
            else
            {
                switch (e.File.Type)
                {
                    case FileType.File:
                        {
                            if (e.OriginalContainer == FileModel.Instance.ActiveFileContainer)
                            {
                                listView_FileList.Items[e.OriginalFileName].Remove();
                            }

                            break;
                        }
                }
            }
        }

        private void FileManager_FileAdded(object sender, CommonFileActionEventArgs e)
        {
            if (ApplicationRuntimeSettings.ShowAllFilesInSubfolder)
            {
                RefreshList();
            }
            else
            {
                if (ShouldInList(e.File))
                {
                    FileListItem item = new FileListItem(e.File);
                    listView_FileList.Items.Add(item);
                    item.ApplyIcon(e.File);
                    item.EnsureVisible();
                }
            }
        }

        private void Instance_FileInformationChanged(object sender, CommonFileActionEventArgs e)
        {
            if (ShouldInList(e.File))
            {
                FileListItem item = listView_FileList.Items[e.File.FullName] as FileListItem;

                if (item != null)
                {
                    item.UpdateData(item.File);
                }
            }
        }

        private void Instance_AfterFileActive(object sender, AfterFileActiveEventArgs e)
        {
            if (e.Item.Type == FileType.FileContainer)
            {
                ShowFiles((IFileContainer)e.Item, false);
            }
        }

        private void listView_FileList_ItemActivate(object sender, EventArgs e)
        {
            if (listView_FileList.SelectedItems.Count > 0)
            {
                ListViewItem item = listView_FileList.SelectedItems[0];
                LunchFile(item);
            }
        }

        private void toolStripMenuItem_Lunch_Click(object sender, EventArgs e)
        {
            foreach (ListViewItem item in listView_FileList.SelectedItems)
            {
                LunchFile(item);
            }
        }

        private void toolStripMenuItem_Count_Click(object sender, EventArgs e)
        {
            String[] fileNames = new String[listView_FileList.SelectedItems.Count];
            int index = 0;

            foreach (ListViewItem item in listView_FileList.SelectedItems)
            {
                fileNames[index++] = item.Name;
            }

            FileModel.Instance.CountFile(false, fileNames);
        }

        private void toolStripMenuItem_ShowCountResult_Click(object sender, EventArgs e)
        {
            if (listView_FileList.SelectedItems.Count > 0)
            {
                ListViewItem item = listView_FileList.SelectedItems[0] as ListViewItem;
                IFile file = FileModel.Instance.FileManager.GetFile(item.Name);
                ApplicationManager.Current.MainForm.ShowFileInformation(file);
            }
        }

        private void listView_FileList_KeyDown(object sender, KeyEventArgs e)
        {
            switch (e.KeyCode)
            {
                case Keys.Delete:
                    {
                        RemoveSelectedFiles();
                        break;
                    }
                case Keys.A:
                    {
                        if (e.Control)
                        {
                            SelectAllItems();
                        }
                        break;
                    }
            }
        }

        private void toolStripMenuItem_RemoveSelectedFiles_Click(object sender, EventArgs e)
        {
            RemoveSelectedFiles();
        }

        private void toolStripMenuItem_RemoveAll_Click(object sender, EventArgs e)
        {
            RemoveAllFiles();
        }

        private void listView_FileList_ItemDrag(object sender, ItemDragEventArgs e)
        {
            if (e.Button == MouseButtons.Left)
            {
                IFile file = null;
                List<IFile> files = new List<IFile>();

                foreach (ListViewItem item in listView_FileList.SelectedItems)
                {
                    file = FileModel.Instance.FileManager.GetFile(item.Name);
                    files.Add(file);
                }

                try
                {
                    DoDragDrop(files, DragDropEffects.Move);
                }
                catch (ArgumentNullException argNullEx)
                {
                    ApplicationDiagnostics.LogException(argNullEx);
                    ApplicationDiagnostics.ShowMessage(
                        Resources.Resource.GetLocalizedString("Text_CannotMoveFiles"),
                        Resources.Resource.GetLocalizedString("Text_CannotFindTarget"));
                }
            }
        }
    }
}
