using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Collections;
using LineCounter.Files;
using LineCounter.Application;

namespace LineCounter.Application.UI
{
    public partial class NavigationPane : UserControl
    {
        private TreeNodeDictionary _treeNodes;
        private TreeNode _currentNode;
        private FileImporter _fileImporter;


        /// <summary>
        /// Initializes a new instance of the <see cref="T:NavigationPane"/> class.
        /// </summary>
        public NavigationPane()
        {
            _treeNodes = new TreeNodeDictionary();

            InitializeComponent();

            treeView_FolderList.ImageKey = "FolderClosed";
            treeView_FolderList.SelectedImageKey = "FolderOpen";

            BindEvent();
        }


        [Category("Behavior")]
        public ImageList ImageList
        {
            get { return treeView_FolderList.ImageList; }
            set
            {
                treeView_FolderList.ImageList = value;
            }
        }

        [Category("Behavior")]
        public FileImporter FileImporter
        {
            get
            {
                if (_fileImporter == null)
                {
                    _fileImporter = new FileImporter();
                }

                return _fileImporter;
            }
            set { _fileImporter = value; }
        }

        private TreeNodeDictionary TreeNodes
        {
            get { return _treeNodes; }
        }

        private TreeNode RootNode
        {
            get
            {
                TreeNode rootNode;
                TreeNodes.TryGetValue(FileModel.Instance.FileManager.Root.FullName, out rootNode);

                return rootNode;
            }
        }

        /// <summary>
        /// Gets or sets the node that user want to operate on.
        /// </summary>
        /// <value>The current node.</value>
        private TreeNode CurrentNode
        {
            get
            {
                return _currentNode == null ? RootNode : _currentNode;
            }
            set
            {
                _currentNode = value;
            }
        }

        /// <summary>
        /// Gets the current folder name assicoat with context menu operations.
        /// </summary>
        private String CurrentFolderName
        {
            get { return CurrentNode.Name; }
        }

        public String SelectedFolderName
        {
            get { return GetSelectedNode().Name; }
        }


        private void BindEvent()
        {
            FileModel.Instance.FileActiveded += new EventHandler<AfterFileActiveEventArgs>(Instance_AfterFileActive);
            FileModel.Instance.FileManager.FileAdded += new EventHandler<CommonFileActionEventArgs>(FileManager_FileAdded);
            FileModel.Instance.FileManager.FileMoved += new EventHandler<FileMoveEventArgs>(FileManager_FileMoved);
            FileModel.Instance.FileManager.FileRemoved += new EventHandler<FileMoveEventArgs>(FileManager_FileRemoved);
            FileModel.Instance.FileInformationChanged += new EventHandler<CommonFileActionEventArgs>(FileManager_FileInformationChanged);
        }

        private void Instance_AfterFileActive(object sender, AfterFileActiveEventArgs e)
        {
            if (e.Item.Type == FileType.FileContainer)
            {
                TreeNode node = TreeNodes[e.Item.FullName];
                treeView_FolderList.SelectedNode = node;
            }
        }

        private void FileManager_FileRemoved(object sender, FileMoveEventArgs e)
        {
            if (e.File.Type == FileType.FileContainer)
            {
                if (((IFileContainer)e.File).ContainerType == FileContainerType.Root)
                {
                    RemoveNode(RootNode);
                }
                else
                {
                    String nodeName = System.IO.Path.Combine(e.OriginalContainer.FullName, e.File.Name);
                    TreeNode node = TreeNodes[nodeName];

                    RemoveNode(node);
                }

                UpdateNode(e.OriginalContainer, true);
            }
        }

        private void FileManager_FileMoved(object sender, FileMoveEventArgs e)
        {
            if (e.File.Type == FileType.FileContainer)
            {
                String nodeName = System.IO.Path.Combine(e.OriginalContainer.FullName, e.File.Name);
                TreeNode node = TreeNodes[nodeName];

                RemoveNode(node);

                treeView_FolderList.BeginUpdate();
                UpdateNode((IFileContainer)e.OriginalContainer, true);
                TreeNode addedNode = AddNode((IFileContainer)e.File);
                UpdateNode((IFileContainer)e.File.Container, true);
                treeView_FolderList.EndUpdate();

                addedNode.EnsureVisible();
            }
        }

        private void FileManager_FileAdded(object sender, CommonFileActionEventArgs e)
        {
            if (e.File.Type == FileType.FileContainer)
            {
                treeView_FolderList.BeginUpdate();
                TreeNode addedNode = AddNode((IFileContainer)e.File);
                treeView_FolderList.EndUpdate();

                addedNode.EnsureVisible();
            }
        }

        private void FileManager_FileInformationChanged(object sender, CommonFileActionEventArgs e)
        {
            if (e.File.Type == FileType.FileContainer)
            {
                String nodeName = e.File.FullName;
                TreeNode node = TreeNodes[nodeName];

                treeView_FolderList.BeginUpdate();
                UpdateNode(node, (IFileContainer)e.File, false);
                treeView_FolderList.EndUpdate();
            }
        }


        /// <summary>
        /// Adds the node to tree view.
        /// </summary>
        private void AddNode(TreeNode node)
        {
            String parentName = TreeNodeDictionary.GetParentName(node.Name);

            if (StringHelper.MaskNullString(parentName).Length == 0)
            {
                treeView_FolderList.Nodes.Add(node);
            }
            else
            {
                TreeNode parentNode = TreeNodes[parentName];

                parentNode.Nodes.Add(node);
            }

            TreeNodes.Add(node);
        }

        /// <summary>
        /// Creates tree nodes from file container and adds them to the tree view.
        /// </summary>
        private TreeNode AddNode(IFileContainer fileContainer)
        {
            TreeNode result = new TreeNode(fileContainer.DisplayName);
            UpdateNode(result, fileContainer, true);
            //result.Name = fileContainer.FullName;
            //result.Text = fileContainer.DisplayName;

            String parentName = fileContainer.Container == null ? null : fileContainer.Container.FullName;

            TreeNode parentNode = null;

            if (parentName != null)
            {
                TreeNodes.TryGetValue(parentName, out parentNode);
            }

            TreeNodes.Add(fileContainer.FullName, result);

            foreach (IFileContainer container in fileContainer.SubContainers.Values)
            {
                TreeNode subNode = AddNode(container);
            }

            if (parentNode != null)
            {
                parentNode.Nodes.Add(result);
            }
            else if (fileContainer.ContainerType == FileContainerType.Root)
            {
                treeView_FolderList.Nodes.Add(result);
            }


            return result;
        }

        private void RemoveNode(TreeNode node)
        {
            TreeNodes.Remove(node.Name);
            node.Remove();
        }

        private void UpdateNode(IFileContainer fileContainer, bool topContainerOnly)
        {
            TreeNode node = TreeNodes[fileContainer.FullName];
            UpdateNode(node, fileContainer, topContainerOnly);
        }

        private void UpdateNode(TreeNode node, IFileContainer fileContainer, bool topContainerOnly)
        {
            node.Name = fileContainer.FullName;
            node.Text = fileContainer.DisplayName;

            if (fileContainer.Counted)
            {
                if (fileContainer.CounterResult.Lines > 0)
                {
                    node.ForeColor = SystemColors.ControlText;
                    node.Text = String.Format("{0} ({1})", node.Text, fileContainer.CounterResult.Lines);
                    node.Expand();
                }
                else if(fileContainer != FileModel.Instance.FileManager.Root)
                {
                    node.ForeColor = SystemColors.GrayText;
                }
            }

            if (!topContainerOnly)
            {
                foreach (IFileContainer c in fileContainer.SubContainers.Values)
                {
                    UpdateNode(c, false);
                }
            }

            node.EnsureVisible();
        }

        /// <summary>
        /// Refreshes the folder list by the specific root's data.
        /// </summary>
        private void RefreshFolderList(RootFileContainer root)
        {
            treeView_FolderList.Nodes.Clear();
            TreeNodes.Clear();

            treeView_FolderList.BeginUpdate();
            TreeNode rootNode = AddNode(root);
            treeView_FolderList.EndUpdate();
        }

        /// <summary>
        /// Gets the tree node the pointer points to, the root node will be return if
        /// no node found.
        /// </summary>
        private TreeNode GetTreeNodeAt(Point point)
        {
            // Get the node item dropped in
            TreeNode result = treeView_FolderList.GetNodeAt(treeView_FolderList.PointToClient(point));

            // Cannot find the node, set the node as root node
            result = result == null ? treeView_FolderList.Nodes[RootFileContainer.RootContainerName] : result;

            return result;
        }

        private TreeNode GetSelectedNode()
        {
            return treeView_FolderList.SelectedNode == null ? RootNode : treeView_FolderList.SelectedNode;
        }

        private void RemoveSelectedFolder(String folderName)
        {
            IFileContainer container = FileModel.Instance.FileManager.GetContainer(folderName);

            try
            {
                FileModel.Instance.FileManager.RemoveFileContainer(container);
            }
            catch (ArgumentNullException argNullEx)
            {
                ApplicationDiagnostics.LogException(argNullEx);

                // Container name not found.
                ApplicationDiagnostics.ShowMessage(
                    Resources.Resource.GetLocalizedString("Text_CannotRemoveFolder"),
                    Resources.Resource.GetLocalizedString("Text_RemoveFolderError", folderName));
            }
            catch (FileContainerNotFoundException fcNotFoundEx)
            {
                ApplicationDiagnostics.LogException(fcNotFoundEx);

                // Container name not found.
                ApplicationDiagnostics.ShowMessage(
                    Resources.Resource.GetLocalizedString("Text_CannotRemoveFolder"),
                    Resources.Resource.GetLocalizedString("Text_CannotFindFolder", fcNotFoundEx.FileDisplayName));
            }
            catch (FileContainerTypeNotAcceptableException fcTypeNotAcceptableEx)
            {
                ApplicationDiagnostics.LogException(fcTypeNotAcceptableEx);

                // Container name not found.
                ApplicationDiagnostics.ShowMessage(
                    Resources.Resource.GetLocalizedString("Text_CannotRemoveFolder"),
                    Resources.Resource.GetLocalizedString("Text_CannotRemoveFolder_GeneralMessage", fcTypeNotAcceptableEx.FileDisplayName));
            }
        }


        private void treeView_FolderList_AfterSelect(object sender, TreeViewEventArgs e)
        {
            CurrentNode = e.Node;
            String folderName = e.Node.Name;
            IFile file = FileModel.Instance.FileManager.GetContainer(folderName);

            FileModel.Instance.ActiveFile(file);
        }

        private void treeView_FolderList_ItemDrag(object sender, ItemDragEventArgs e)
        {
            if (e.Button == MouseButtons.Left)
            {
                TreeNode node = (TreeNode)e.Item;

                if (node.Name != FileModel.Instance.FileManager.Root.FullName)
                {
                    IFileContainer fileContainer = FileModel.Instance.FileManager.GetContainer(node.Name);

                    try
                    {
                        DoDragDrop(fileContainer, DragDropEffects.Move);
                    }
                    catch (ArgumentNullException argNullEx)
                    {
                        ApplicationDiagnostics.LogException(argNullEx);
                        ApplicationDiagnostics.ShowMessage(
                            Resources.Resource.GetLocalizedString("Text_CannotMoveFolder"),
                            Resources.Resource.GetLocalizedString("Text_CannotFindFolder", node.Name));
                    }
                }
            }
        }

        private void treeView_FolderList_DragEnter(object sender, DragEventArgs e)
        {
            dragDropHelper.StartNewDrag(e.Data);

            switch (dragDropHelper.DropType)
            {
                case DropType.None:
                    e.Effect = DragDropEffects.None;
                    break;
                case DropType.InternalFile:
                    e.Effect = DragDropEffects.Move;
                    break;
                case DropType.SystemFile:
                    e.Effect = DragDropEffects.Copy;
                    break;
                case DropType.Unknown:
                    e.Effect = DragDropEffects.None;
                    break;
            }
        }

        private void treeView_FolderList_DragDrop(object sender, DragEventArgs e)
        {
            TreeNode dropOnNode = GetTreeNodeAt(new Point(e.X, e.Y));
            IFileContainer targetContainer = FileModel.Instance.FileManager.GetContainer(dropOnNode.Name);

            // Store the data stored in drag drop helper, in case they would be changed
            // when being operated.
            DropType dropType = dragDropHelper.DropType;
            Object dragData = dragDropHelper.GetDraggingData();

            // End current drag
            dragDropHelper.EndCurrentDrag();

            switch (dropType)
            {
                case DropType.None:
                    { break; }
                case DropType.InternalFile:
                    {
                        IList<IFile> internalFiles = (IList<IFile>)dragData;

                        foreach (IFile internalFile in internalFiles)
                        {
                            try
                            {
                                FileModel.Instance.FileManager.MoveFile(internalFile, targetContainer);
                            }
                            catch (FileOperationNotSupportedException fileOperationNotSupportedEx)
                            {
                                ApplicationDiagnostics.LogException(fileOperationNotSupportedEx);
                            }
                        }

                        break;
                    }
                case DropType.SystemFile:
                    {
                        String[] fileNames = (String[])dragData;

                        FileModel.Instance.ImportFile(fileNames, targetContainer.FullName);

                        break;
                    }
                case DropType.Unknown:
                    { break; }
            }
        }

        private void importFolderToolStripMenuItem_Click(object sender, EventArgs e)
        {
            FileImporter.RequestImportFolder(CurrentFolderName);
        }

        private void importFileToolStripMenuItem_Click(object sender, EventArgs e)
        {
            FileImporter.RequestImportFile(CurrentFolderName);
        }

        private void refreshToolStripMenuItem_Click(object sender, EventArgs e)
        {
            RefreshFolderList(FileModel.Instance.FileManager.Root);
        }

        private void removeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            RemoveSelectedFolder(CurrentNode.Name);
        }

        private void treeView_FolderList_AfterLabelEdit(object sender, NodeLabelEditEventArgs e)
        {
            String name = e.Node.Name;
            IFileContainer folder = FileModel.Instance.FileManager.GetContainer(name);

            folder.DisplayName = e.Label == null ? folder.DisplayName : e.Label;
        }

        private void renameToolStripMenuItem_Click(object sender, EventArgs e)
        {
            CurrentNode.BeginEdit();
        }

        private void treeView_FolderList_MouseClick(object sender, MouseEventArgs e)
        {

        }

        private void contextMenuStrip_Nav_Opening(object sender, CancelEventArgs e)
        {
            CurrentNode = GetTreeNodeAt(new Point(contextMenuStrip_Nav.Left, contextMenuStrip_Nav.Top));
        }

        private void toolStripMenuItem_FileInfo_Click(object sender, EventArgs e)
        {
            IFileContainer container = FileModel.Instance.FileManager.GetContainer(CurrentFolderName);

            ApplicationManager.Current.MainForm.ShowFileInformation(container);
        }

        private void treeView_FolderList_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete)
            {
                RemoveSelectedFolder(SelectedFolderName);
            }
        }

        private void toolStripMenuItem_AddFolder_Click(object sender, EventArgs e)
        {
            String folderText = Resources.Resource.GetLocalizedString("Text_NewFolder");
            IFileContainer container = FileModel.Instance.FileManager.GetContainer(CurrentNode.Name);

            Folder newFolder = new Folder(DateTime.Now.Ticks.ToString());
            newFolder.DisplayName = folderText;

            FileModel.Instance.FileManager.AddContainer(newFolder, container.FullName);

            TreeNodes[newFolder.FullName].BeginEdit();
        }
    }
}