using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using LineCounter.Resources;

namespace LineCounter.Application.UI
{
    public partial class OptionsForm : Form
    {
        private const String GeneralPageTag = "General";
        private const String FileFilterPageTag = "FileFilter";
        private const String FolderFilterPageTag = "FolderFilter";
        private const String NoPageTag = "NoPage";
        private const String CounterOptionsParentTagHeader = "CounterOptionsParent_";
        private const String CounterOptionsTagHeader = "CounterOptions_";
        private OptionsPages.OptionsPageDictionary _pages;
        private TreeNode counterNode;
        private Boolean cancelHandled;


        /// <summary>
        /// Initializes a new instance of the <see cref="T:OptionsForm"/> class.
        /// </summary>
        public OptionsForm()
        {
            ApplicationRuntimeSettings.SettingsReseted += new EventHandler(ApplicationRuntimeSettings_SettingsReseted);

            InitializeComponent();

            LocalizeUI();

            AppendPredefinedNodes();

            AppendCountersOptionsNodes();

            _pages = new LineCounter.Application.UI.OptionsPages.OptionsPageDictionary();

            treeView.ExpandAll();
        }

        private void ApplicationRuntimeSettings_SettingsReseted(object sender, EventArgs e)
        {
            foreach (LineCounter.Application.UI.OptionsPages.BasePage page in Pages.Values)
            {
                page.ResetOptions();
            }
        }


        private OptionsPages.OptionsPageDictionary Pages
        {
            get { return _pages; }
        }


        private void AppendPredefinedNodes()
        {
            TreeNode generalNode = new TreeNode(Resource.GetLocalizedString("Text_General"));
            TreeNode fileFilterNode = new TreeNode(Resource.GetLocalizedString("Text_FileFilter"));
            TreeNode folderFilterNode = new TreeNode(Resource.GetLocalizedString("Text_FolderFilter"));
            TreeNode counterNode = new TreeNode(Resource.GetLocalizedString("Text_CounterOptions"));

            generalNode.Tag = GeneralPageTag;
            generalNode.Nodes.Add(fileFilterNode);
            generalNode.Nodes.Add(folderFilterNode);
            fileFilterNode.Tag = FileFilterPageTag;
            folderFilterNode.Tag = FolderFilterPageTag;
            counterNode.Tag = NoPageTag;
            this.counterNode = counterNode;

            treeView.Nodes.Add(generalNode);
            treeView.Nodes.Add(counterNode);
        }

        private void AppendCountersOptionsNodes()
        {
            foreach (CounterItem counterItem in CounterManager.Current.CounterItems)
            {
                if (counterItem.HasEditableOptions)
                {
                    TreeNode node = new TreeNode(counterItem.Name);
                    node.Tag = CounterOptionsParentTagHeader + counterItem.Id;

                    foreach (String nodeName in counterItem.PropertyDescriptor.GetOptionsPagesNames())
                    {
                        TreeNode subNode = new TreeNode(nodeName);
                        subNode.Tag = CounterOptionsTagHeader + nodeName;

                        node.Nodes.Add(subNode);
                    }

                    counterNode.Nodes.Add(node);
                }
            }
        }

        private void LocalizeUI()
        {
            button_Cancel.Text = Resource.GetLocalizedString("Text_HotKey_Cancel");
            button_Ok.Text = Resource.GetLocalizedString("Text_HotKey_Ok");
            this.Text = Resource.GetLocalizedString("Text_Options");
        }

        /// <summary>
        /// Gets options page via page tag.
        /// </summary>
        private OptionsPages.BasePage GetPage(String pageTag, String parentTag)
        {
            OptionsPages.BasePage page = null;
            bool toAdd = false;

            // Determain whether the page is already created
            if (!Pages.TryGetValue(pageTag, out page))
            {
                // Check if the page is counters options page
                if (pageTag.IndexOf(CounterOptionsTagHeader) == 0)
                {
                    // The page is counter options page, get it
                    // from the counter's property descriptor

                    // Gets the counter's name
                    String counterName = parentTag.Substring(CounterOptionsParentTagHeader.Length);
                    // Gets the page's name
                    String pageName = pageTag.Substring(CounterOptionsTagHeader.Length);
                    // Gets the counter's property descriptor
                    LineCounter.CounterOptionsDescriptorBase descriptor = CounterManager.Current.GetCounter(counterName).PropertyDescriptor;
                    // Gets the page
                    page = descriptor.GetOptionsPage(pageName);
                    page.Name = pageTag;

                    toAdd = true;
                }
                else
                {
                    // The page is application pre-defined, and hasn't been initialized.
                    // Now try to create it.
                    switch (pageTag)
                    {
                        case GeneralPageTag:
                            {
                                page = new LineCounter.Application.UI.OptionsPages.General();
                                page.Name = GeneralPageTag;
                                toAdd = true;
                                break;
                            }
                        case FileFilterPageTag:
                            {
                                page = new LineCounter.Application.UI.OptionsPages.FileFilterPage();
                                page.Name = FileFilterPageTag;
                                toAdd = true;
                                break;
                            }
                        case FolderFilterPageTag:
                            {
                                page = new LineCounter.Application.UI.OptionsPages.FolderFilterPage();
                                page.Name = FolderFilterPageTag;
                                toAdd = true;
                                break;
                            }
                        case NoPageTag:
                            {
                                goto default;
                            }
                        default:
                            {
                                page = null;
                                break;
                            }
                    }
                }
            }

            if (toAdd)
            {
                Pages.Add(page);
            }

            return page;
        }

        /// <summary>
        /// Shows the options page.
        /// </summary>
        /// <remarks>The previous page won't be changed if the page argument is null.</remarks>
        /// <param name="page">The page to show.</param>
        private void ShowPage(LineCounter.Application.UI.OptionsPages.BasePage page)
        {
            if (page != null && !panel_Options.Controls.Contains(page))
            {
                panel_Options.Controls.Clear();
                panel_Options.Controls.Add(page);
            }
        }

        private void treeView_AfterSelect(object sender, TreeViewEventArgs e)
        {
            String parentTag = null;

            if (e.Node.Parent != null)
            {
                parentTag = (String)e.Node.Parent.Tag;
            }

            LineCounter.Application.UI.OptionsPages.BasePage page = GetPage((String)e.Node.Tag, parentTag);
            ShowPage(page);
        }

        private void button_Ok_Click(object sender, EventArgs e)
        {
            cancelHandled = true;

            foreach (OptionsPages.BasePage page in Pages.Values)
            {
                page.ApplyChanges();
            }

            this.Close();
        }

        private void button_Cancel_Click(object sender, EventArgs e)
        {
            cancelHandled = true;

            foreach (OptionsPages.BasePage page in Pages.Values)
            {
                page.CancelChanges();
            }

            this.Close();
        }

        protected override void OnFormClosed(FormClosedEventArgs e)
        {
            if (cancelHandled)
            {
                cancelHandled = false;
            }
            else
            {
                this.CancelButton.PerformClick();
            }
        }

        private void button_Reset_Click(object sender, EventArgs e)
        {
            ApplicationRuntimeSettings.ResetAllSettings();
        }
    }
}