using System;
using System.Collections.Generic;
using System.Text;

namespace LineCounter
{
    public enum LineBlockHandleAction
    {
        None = 0,
        Excluded = 1,
        Counted = 2
    }

    /// <summary>
    /// Represents a file element.
    /// </summary>
    public class FileElement : INamedObject
    {
        #region Local Varaiables

        private String _displayName;
        private String _name;
        private String _sourceFile;
        private LineBlockHandleAction _handleAction;
        private FileElementIdentity _identity;
        private FileElementHeader _header;
        private FileElementCollection _elements;
        private FilePosition _elementBlockOpenPosition;
        private FilePosition _elementBlockClosedPosition;
        private FileElement _container;

        #endregion


        public FileElement(FileElementIdentity identity)
        {
            _identity = identity;
            _elements = new FileElementCollection(this);
            _elementBlockOpenPosition = new FilePosition(-1, 0);
            _elementBlockClosedPosition = new FilePosition(-1, 0);
        }


        #region Properties

        /// <summary>
        /// Gets or sets the source file containing the block.
        /// </summary>
        public String SourceFile
        {
            get { return _sourceFile; }
            set { _sourceFile = value; }
        }

        /// <summary>
        /// Gets or sets the action profermed on the block.
        /// </summary>
        public LineBlockHandleAction HandleAction
        {
            get { return _handleAction; }
            set { _handleAction = value; }
        }

        /// <summary>
        /// Gets the identity of the block.
        /// </summary>
        public FileElementIdentity Identity
        {
            get { return _identity; }
        }

        /// <summary>
        /// Gets or sets the block header.
        /// </summary>
        public FileElementHeader Header
        {
            get { return _header; }
            set 
            {
                if (_header != null)
                {
                    _header.Element = null;
                }

                if (value != null)
                {
                    value.Element = this;
                }

                // Refresh display name
                _displayName = null;

                _header = value;
            }
        }

        /// <summary>
        /// Gets sub blocks.
        /// </summary>
        public FileElementCollection Elements
        {
            get { return _elements; }
        }

        /// <summary>
        /// Gets or sets the position where the block opened.
        /// </summary>
        public FilePosition ElementBlockOpenPosition
        {
            get { return _elementBlockOpenPosition; }
            set { _elementBlockOpenPosition = value; }
        }

        /// <summary>
        /// Gets or sets the position where the block closed.
        /// </summary>
        public FilePosition ElementBlockClosedPosition
        {
            get { return _elementBlockClosedPosition; }
            set { _elementBlockClosedPosition = value; }
        }

        /// <summary>
        /// Gets a value indicating whether the block is closed.
        /// </summary>
        public Boolean ElementBlockClosed
        {
            get { return (ElementBlockClosedPosition.IsValidPosition); }
        }

        /// <summary>
        /// Gets or sets the container.
        /// </summary>
        public FileElement Container
        {
            get { return _container; }
            set { _container = value; }
        }

        public FilePosition StartPosition
        {
            get
            {
                switch (Identity.HeaderPosition)
                {
                    case FileElementHeaderPosition.InFrontOfBlock:
                        {
                            if (Header != null)
                            {
                                return Header.StartPosition;
                            }
                            else
                            {
                                return new FilePosition(-1, 0);
                            }
                        }
                    case FileElementHeaderPosition.None:
                        {
                            return ElementBlockOpenPosition;
                        }
                    default:
                        {
                            return ElementBlockOpenPosition;
                        }
                }
            }
        }

        public FilePosition EndPosition
        {
            get { return ElementBlockClosedPosition; }
        }

        #endregion

        #region INamedObject Members

        public String Name
        {
            get { return _name; }
            set { _name = value; }
        }

        public String DisplayName
        {
            get
            {
                if (String.IsNullOrEmpty(_displayName))
                {
                    _displayName = FindElementName();
                }

                return _displayName;
            }
            set { _displayName = value; }
        }

        #endregion

        public String FindElementName()
        {
            if(String.IsNullOrEmpty(Identity.ElementNameKeyword))
            {
                return null;
            }

            switch (Identity.ElementNamePosition)
            {
                case FileElementNamePosition.None:
                    { return null; }
                case FileElementNamePosition.Header:
                    {
                        if (Header != null)
                        {
                            return Header.Keywords[Identity.ElementNameKeyword];
                        }

                        return null;
                    }
                case FileElementNamePosition.BlockStartSymbol:
                    switch (Identity.ElementBlockOpenSymbol.FindMethod)
                    {
                        case FindMethod.RegularExpression:
                            {
                                return null;
                            }
                        default:
                            {
                                return null;
                            }
                    }
                case FileElementNamePosition.BlockCloseSymbol:
                    switch (Identity.ElementBlockOpenSymbol.FindMethod)
                    {
                        case FindMethod.RegularExpression:
                            {
                                return null;
                            }
                        default:
                            {
                                return null;
                            }
                    }
                default:
                    {
                        return null;
                    }
            }
        }
    }
}
