using System;
using System.Collections.Generic;
using System.Text;

namespace LineCounter
{
    public class FileFormat
    {
        #region Local Varaiables

        private Dictionary<String, FileElementIdentity> _identities;
        private Boolean _identitySummaryBuilt;
        private System.Collections.Specialized.StringCollection _nonCodeElementIdentityNames;
        private System.Collections.Specialized.NameValueCollection blockOpenKeywords;
        private System.Collections.Specialized.NameValueCollection blockCloseKeywords;
        private const string rootName = "$root$";

        #endregion

        public FileFormat()
        {
            _nonCodeElementIdentityNames = new System.Collections.Specialized.StringCollection();
            _identities = new Dictionary<string, FileElementIdentity>();
            _identitySummaryBuilt = false;
            blockOpenKeywords = new System.Collections.Specialized.NameValueCollection();
            blockCloseKeywords = new System.Collections.Specialized.NameValueCollection();
        }


        #region Properties

        public Dictionary<String, FileElementIdentity> Identities
        {
            get { return _identities; }
        }

        public Boolean IdentitySummaryBuilt
        {
            get { return _identitySummaryBuilt; }
        }

        public System.Collections.Specialized.StringCollection NonCodeElementIdentityNames
        {
            get { return _nonCodeElementIdentityNames; }
            set { _nonCodeElementIdentityNames = value; }
        }

        #endregion

        public IEnumerable<FileElementIdentity> GetRootLegalElementIdentities()
        {
            List<FileElementIdentity> result = new List<FileElementIdentity>();

            foreach (FileElementIdentity item in Identities.Values)
            {
                if (item.CanBeRootElement)
                {
                    if (item.ContentType == FileElementContentType.MutipleLineStringLiteral || item.ContentType == FileElementContentType.SingleLineStringLiteral)
                    {
                        result.Insert(0, item);
                    }
                    else
                    {
                        result.Add(item);
                    }
                }
            }

            return result;
        }

        public IEnumerable<FileElementIdentity> GetLegalSubElementIdentities(String containerBlockIdentityName)
        {
            List<FileElementIdentity> result = new List<FileElementIdentity>();

            foreach (FileElementIdentity item in Identities.Values)
            {
                if (item.LegalContainerIdentityNames.Contains(containerBlockIdentityName))
                {
                    if (item.ContentType == FileElementContentType.MutipleLineStringLiteral || item.ContentType == FileElementContentType.SingleLineStringLiteral)
                    {
                        result.Insert(0, item);
                    }
                    else
                    {
                        result.Add(item);
                    }
                }
            }

            return result;
        }

        public SearchKeyword GetBlockCloseKeyword()
        {
            return GetBlockCloseKeyword(rootName);
        }

        public SearchKeyword GetBlockCloseKeyword(String containerBlockIdentityName)
        {
            String keyword = null;

            if (containerBlockIdentityName == null)
            {
                containerBlockIdentityName = rootName;
            }

            keyword = blockCloseKeywords[containerBlockIdentityName];

            return new SearchKeyword(keyword, FindMethod.RegularExpression);
        }

        public SearchKeyword GetBlockOpenKeyword()
        {
            return GetBlockOpenKeyword(rootName);
        }

        public SearchKeyword GetBlockOpenKeyword(String containerBlockIdentityName)
        {
            String keyword = null;

            if (containerBlockIdentityName == null)
            {
                containerBlockIdentityName = rootName;
            }

            keyword = blockOpenKeywords[containerBlockIdentityName];

            return new SearchKeyword(keyword, FindMethod.RegularExpression);
        }

        public void BuildIdentitySummary()
        {
            foreach (FileElementIdentity item in Identities.Values)
            {
                if (item.ContentType == FileElementContentType.SingleLineStringLiteral || item.ContentType == FileElementContentType.MutipleLineStringLiteral)
                {
                    _nonCodeElementIdentityNames.Add(item.Name);
                }

                BuildBlockSymbolSummary(item.Name);
            }

            BuildBlockSymbolSummary(rootName);

            _identitySummaryBuilt = true;
        }

        private void BuildBlockSymbolSummary(String itemName)
        {
            List<String> blockOpenKeywordsList = new List<String>();
            List<String> blockCloseKeywordsList = new List<String>();

            IEnumerable<FileElementIdentity> identities = null;

            if (itemName == rootName)
            {
                identities = GetRootLegalElementIdentities();
            }
            else
            {
                identities = GetLegalSubElementIdentities(itemName);
            }

            foreach (FileElementIdentity item in identities)
            {
                if (!blockOpenKeywordsList.Contains(item.ElementBlockOpenSymbol.Keyword))
                {
                    blockOpenKeywordsList.Add(item.ElementBlockOpenSymbol.Keyword);
                }

                if (!blockCloseKeywordsList.Contains(item.ElementBlockClosedSymbol.Keyword))
                {
                    blockCloseKeywordsList.Add(item.ElementBlockClosedSymbol.Keyword);
                }
            }

            StringBuilder blockOpenKeywordsRegexBuilder = new StringBuilder();
            StringBuilder blockCloseKeywordsRegexBuilder = new StringBuilder();

            foreach (String keyword in blockOpenKeywordsList)
            {
                blockOpenKeywordsRegexBuilder.AppendFormat("({0})|", keyword);
            }

            foreach (String keyword in blockCloseKeywordsList)
            {
                blockCloseKeywordsRegexBuilder.AppendFormat("({0})|", keyword);
            }

            if (blockCloseKeywordsRegexBuilder.Length > 0)
            {
                blockCloseKeywordsRegexBuilder.Remove(blockCloseKeywordsRegexBuilder.Length - 1, 1);
            }

            if (blockOpenKeywordsRegexBuilder.Length > 0)
            {
                blockOpenKeywordsRegexBuilder.Remove(blockOpenKeywordsRegexBuilder.Length - 1, 1);
            }

            if (blockCloseKeywordsRegexBuilder.Length > 0)
            {
                blockCloseKeywords.Add(itemName, blockCloseKeywordsRegexBuilder.ToString());
            }

            if (blockOpenKeywordsRegexBuilder.Length > 0)
            {
                blockOpenKeywords.Add(itemName, blockOpenKeywordsRegexBuilder.ToString());
            }
        }

    }
}
