using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Runtime.InteropServices;
using System.IO;
using System.Collections.Specialized;

namespace LineCounter
{
    [CLSCompliant(false)]
    [StructLayout(LayoutKind.Sequential)]
    public struct SHFILEINFO
    {
        internal IntPtr hIcon;
        internal int iIcon;
        internal uint dwAttributes;
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 260)]
        internal string szDisplayName;
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 80)]
        internal string szTypeName;
    };

    public static class FileHelper
    {
        public struct ShellFileInfo
        {
            private String _displayName;
            private String _typeName;
            private Icon _icon;
            private Int32 _systemImageListIndex;


            public String DisplayName
            {
                get { return _displayName; }
                set { _displayName = value; }
            }

            public String TypeName
            {
                get { return _typeName; }
                set { _typeName = value; }
            }

            public Icon Icon
            {
                get { return _icon; }
                set { _icon = value; }
            }

            public Int32 SystemImageListIndex
            {
                get { return _systemImageListIndex; }
                set { _systemImageListIndex = value; }
            }
        }

        public enum IconSize
        {
            None = 0x2,
            Small = (Int32)NativeMethods.SHGFI_SMALLICON,
            Large = (Int32)NativeMethods.SHGFI_LARGEICON
        }

        private static class NativeMethods
        {
            public const uint SHGFI_ICON = 0x100;         // Gets the icon 
            public const uint SHGFI_DISPLAYNAME = 0x200;  // Gets the Display name
            public const uint SHGFI_TYPENAME = 0x400;     // Gets the type name
            public const uint SHGFI_LARGEICON = 0x0;      // Large icon
            public const uint SHGFI_SMALLICON = 0x1;      // Small icon

            [DllImport("shell32.dll")]
            public static extern IntPtr SHGetFileInfo(string pszPath, uint dwFileAttributes, ref SHFILEINFO psfi, uint cbSizeFileInfo, uint uFlags);

            public static SHFILEINFO GetFileInfo(string path, IconSize iconSize)
            {
                SHFILEINFO info = new SHFILEINFO();
                IntPtr icon;

                if (iconSize == IconSize.None)
                {
                    SHGetFileInfo(path, 0, ref info, (uint)Marshal.SizeOf(info), SHGFI_TYPENAME | SHGFI_DISPLAYNAME);
                }
                else
                {
                    icon = SHGetFileInfo(path, 0, ref info, (uint)Marshal.SizeOf(info), SHGFI_ICON | SHGFI_TYPENAME | SHGFI_DISPLAYNAME | (uint)iconSize);
                }

                return info;
            }
        }


        public static String Combine(String path1, String path2)
        {
            return System.IO.Path.Combine(path1, path2);
        }

        public static String MapPath(String relativePath)
        {
            if (relativePath == null)
            {
                throw new ArgumentNullException("relativePath");
            }

            if (relativePath.IndexOf('\\') == 0)
            {
                relativePath = relativePath.Substring(1);
            }

            return System.IO.Path.Combine(Environment.CurrentDirectory, relativePath);
        }

        public static String GetFileName(String fullName)
        {
            if (fullName == null)
            {
                throw new ArgumentNullException("fullName");
            }

            return System.IO.Path.GetFileName(fullName);
        }

        public static String GetExtension(String fileName)
        {
            if (fileName == null)
            {
                throw new ArgumentNullException("fileName");
            }

            return System.IO.Path.GetExtension(fileName);
        }

        [CLSCompliant(false)]
        public static ShellFileInfo GetFileInfo(String fileName, IconSize size)
        {
            SHFILEINFO info = NativeMethods.GetFileInfo(fileName, size);

            ShellFileInfo result = new ShellFileInfo();

            result.DisplayName = info.szDisplayName;

            try
            {
                result.Icon = Icon.FromHandle(info.hIcon);
            }
            catch
            {
                result.Icon = null;
            }

            result.SystemImageListIndex = info.iIcon;
            result.TypeName = info.szTypeName;

            return result;
        }

        public static StringCollection GetLocalFolders(String path, SearchPatternCollection searchPatterns, SearchOption searchOption)
        {
            if (StringHelper.IsNullOrEmpty(path))
            {
                throw new ArgumentNullException("path");
            }

            if (searchPatterns == null)
            {
                throw new ArgumentNullException("searchPatterns");
            }

            StringCollection result = new StringCollection();

            if (searchPatterns.Contains("*.*") || searchPatterns.Contains("*"))
            {
                result.AddRange(Directory.GetDirectories(path));
            }
            else
            {
                foreach (String pattern in searchPatterns)
                {
                    if (result.Count == 0)
                    {
                        result.AddRange(Directory.GetDirectories(path, pattern, searchOption));
                    }
                    else
                    {
                        foreach (String file in Directory.GetDirectories(path, pattern, searchOption))
                        {
                            if (!result.Contains(file))
                            {
                                result.Add(file);
                            }
                        }
                    }
                }
            }

            return result;
        }

        public static StringCollection GetLocalFolders(String path)
        {
            if (StringHelper.IsNullOrEmpty(path))
            {
                throw new ArgumentNullException("path");
            }

            StringCollection result = new StringCollection();

            result.AddRange(System.IO.Directory.GetDirectories(path));

            return result;
        }

        public static StringCollection GetLocalFiles(String path, SearchPatternCollection searchPatterns, SearchOption searchOption)
        {
            if (StringHelper.IsNullOrEmpty(path))
            {
                throw new ArgumentNullException("path");
            }

            if (searchPatterns == null)
            {
                throw new ArgumentNullException("searchPatterns");
            }

            StringCollection result = new StringCollection();

            if (searchPatterns.Contains("*.*") || searchPatterns.Contains("*"))
            {
                result.AddRange(Directory.GetFiles(path));
            }
            else
            {
                foreach (String pattern in searchPatterns)
                {
                    if (result.Count == 0)
                    {
                        result.AddRange(Directory.GetFiles(path, pattern, searchOption));
                    }
                    else
                    {
                        foreach (String file in Directory.GetFiles(path, pattern, searchOption))
                        {
                            if (!result.Contains(file))
                            {
                                result.Add(file);
                            }
                        }
                    }
                }
            }

            return result;
        }

        public static StringCollection GetLocalFiles(String path)
        {
            if (StringHelper.IsNullOrEmpty(path))
            {
                throw new ArgumentNullException(path);
            }

            StringCollection result = new StringCollection();

            result.AddRange(System.IO.Directory.GetFiles(path));

            return result;
        }
    }
}
