using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace LineCounter
{
    public class FileOutlineGenerator
    {
        private FileElementCollection _elements;
        private FileFormat _fileFormat;
        private Stack<FileElement> _elementStack;
        private Int32 startLine;
        private Int32 lineCount;


        public FileOutlineGenerator(FileFormat format)
        {
            _fileFormat = format;
            _elementStack = new Stack<FileElement>();
            _elements = new FileElementCollection();
        }


        private FileElementCollection Elements
        {
            get { return _elements; }
        }

        public FileFormat FileFormat
        {
            get { return _fileFormat; }
        }

        private Stack<FileElement> ElementStack
        {
            get
            {
                if (_elementStack == null)
                {
                    _elementStack = new Stack<FileElement>();
                }

                return _elementStack;
            }
        }


        #region Parse

        private void ParseFile(TextReader reader)
        {
            Reset();

            startLine = 1;

            Int32 lineNumber = 1;
            String currentLine = reader.ReadLine();

            while (currentLine != null)
            {
                ParseLine(currentLine, lineNumber++);
                currentLine = reader.ReadLine();
            }

            lineCount = lineNumber - startLine + 1;
        }

        private void ParseLine(string line, int lineNumber)
        {
            if (!FileFormat.IdentitySummaryBuilt)
            {
                FileFormat.BuildIdentitySummary();
            }

            Int32 startIndex = 0;

            // Find the block open position of the block on the top of stack if it don't have a block open position
            if (ElementStack.Count > 0 && !ElementStack.Peek().ElementBlockOpenPosition.IsValidPosition)
            {
                FilePosition containerBlockOpenPosition = StringHelper.SearchKeywordIndex(
                    line,
                    ElementStack.Peek().Identity.ElementBlockOpenSymbol,
                    0,
                    line.Length);

                if (!containerBlockOpenPosition.IsValidPosition)
                {
                    return;
                }

                containerBlockOpenPosition.LineNumber = lineNumber;
                ElementStack.Peek().ElementBlockOpenPosition = containerBlockOpenPosition;

                // Jump over the container block open position, in case of
                // confusing it with no header block with the same open symbol.
                startIndex = containerBlockOpenPosition.Index + containerBlockOpenPosition.Count;
            }

            if (startIndex < line.Length)
            {
                ParseLineSpan(line, 0, line.Length - 1, lineNumber);
            }
        }

        private void ParseLineSpan(String line, Int32 startIndex, Int32 endIndex, Int32 lineNumber)
        {
            Int32 currentStartIndex = startIndex;
            Int32 currentEndIndex = endIndex;

            if (startIndex == -1 || endIndex == -1)
            {
                return;
            }

            while (currentStartIndex <= endIndex)
            {
                // Get current span position and determain whether the parent block is closed
                if (ElementStack.Count > 0)
                {
                    Int32 remainningLevel = 0;
                    //currentStartIndex = startIndex;
                    FilePosition containerBlockClosePosition = new FilePosition(-1, 0);

                    if (ElementStack.Peek().ElementBlockOpenPosition.LineNumber != lineNumber)
                    {
                        containerBlockClosePosition = FindBlockEndPosition(
                        ElementStack.Peek().Identity,
                        line,
                        1,
                        out remainningLevel,
                        currentStartIndex);
                        containerBlockClosePosition.LineNumber = lineNumber;
                    }
                    else if(!ElementStack.Peek().ElementBlockClosed)
                    {
                        containerBlockClosePosition = FindBlockEndPosition(
                        ElementStack.Peek().Identity,
                        line,
                        0,
                        out remainningLevel,
                        currentStartIndex);
                        containerBlockClosePosition.LineNumber = lineNumber;
                    }

                    if (!containerBlockClosePosition.IsValidPosition)
                    {
                        // The top-level block doesn't end in the current line
                        currentEndIndex = endIndex;
                    }
                    else
                    {
                        currentEndIndex = containerBlockClosePosition.Index + containerBlockClosePosition.Count - 1;
                        ElementStack.Peek().ElementBlockClosedPosition = containerBlockClosePosition;
                    }
                }
                else
                {
                    //currentStartIndex = startIndex;
                    currentEndIndex = endIndex;
                }


                IEnumerable<FileElementIdentity> identities = ElementStack.Count > 0 ?
                    FileFormat.GetLegalSubElementIdentities(ElementStack.Peek().Identity.Name) :
                    FileFormat.GetRootLegalElementIdentities();

                List<FileElement> topLevelBlocks = FindTopLevelBlocks(line, currentStartIndex, currentEndIndex - currentStartIndex + 1, identities, lineNumber);

                if (topLevelBlocks.Count > 0)
                {
                    // If no blocks found, the contained code maybe a statement, needn't to do anything.
                    // Now we found something, let's handle them...

                    // Add found blocks into container block
                    topLevelBlocks.ForEach(new Action<FileElement>(AppendToOutline));

                    foreach (FileElement topLevelBlock in topLevelBlocks)
                    {
                        if (topLevelBlock.Identity.CanHaveSubItems)
                        {
                            ElementStack.Push(topLevelBlock);

                            if (!topLevelBlock.ElementBlockClosed)
                            {
                                ParseLineSpan(line,
                                    topLevelBlock.ElementBlockOpenPosition.Index + topLevelBlock.ElementBlockOpenPosition.Count,
                                    topLevelBlock.ElementBlockClosedPosition.Index,
                                    lineNumber);
                            }
                            else
                            {
                                ParseLineSpan(line,
                                    topLevelBlock.ElementBlockOpenPosition.Index + topLevelBlock.ElementBlockOpenPosition.Count,
                                    topLevelBlock.ElementBlockClosedPosition.Index - 1,
                                    lineNumber);
                                ElementStack.Pop();
                            }
                        }
                    }
                }
                else
                {
                    if (ElementStack.Count > 0 &&
                        ElementStack.Peek().ElementBlockClosed &&
                        ElementStack.Peek().ElementBlockOpenPosition.LineNumber != lineNumber)
                    {
                        ElementStack.Pop();
                    }
                }

                currentStartIndex = currentEndIndex + 1;
            }
        }

        protected List<FileElement> FindTopLevelBlocks(String trimedLine, Int32 startIndex, Int32 count, IEnumerable<FileElementIdentity> identities, Int32 lineNumber)
        {
            List<FileElement> result = new List<FileElement>();
            FileElement blockFound = null;
            FileElementHeader headerFound = null;
            Int32 blockStartIndex = -1, headerEndIndex = -1, endIndex = startIndex + count, findEndIndex = -1;

            if (endIndex >= trimedLine.Length)
            {
                endIndex = trimedLine.Length - 1;
            }

            while (startIndex <= endIndex)
            {
                blockFound = null;

                if (ElementStack.Count > 0)
                {
                    //findEndIndex = StringHelper.SearchKeywordIndex(trimedLine, FileFormat.GetBlockCloseKeyword(ElementStack.Peek().Name), startIndex, count).Index;
                    findEndIndex = StringHelper.SearchKeywordIndex(trimedLine, FileFormat.GetBlockCloseKeyword(ElementStack.Peek().Name), startIndex, endIndex - startIndex + 1).Index;
                }
                else
                {
                    //findEndIndex = StringHelper.SearchKeywordIndex(trimedLine, FileFormat.GetBlockCloseKeyword(), startIndex, count).Index;
                    findEndIndex = StringHelper.SearchKeywordIndex(trimedLine, FileFormat.GetBlockCloseKeyword(), startIndex, endIndex - startIndex + 1).Index;
                }

                if (findEndIndex < 0)
                {
                    findEndIndex = endIndex;
                }

                foreach (FileElementIdentity identity in identities)
                {
                    if (startIndex > endIndex || startIndex >= trimedLine.Length)
                    {
                        break;
                    }

                    // Find block header.
                    switch (identity.HeaderPosition)
                    {
                        case FileElementHeaderPosition.None:
                            {
                                headerEndIndex = startIndex - 1;
                                blockStartIndex = startIndex;
                                break;
                            }
                        case FileElementHeaderPosition.InFrontOfBlock:
                            {
                                headerFound = FindElementHeader(trimedLine, startIndex, findEndIndex - startIndex + 1, identity);
                                //headerFound = FindElementHeader(trimedLine, startIndex, endIndex - startIndex + 1, identity);

                                // Cannot find header, try next header identity.
                                if (headerFound == null)
                                {
                                    continue;
                                }
                                else
                                {
                                    headerFound.StartPosition = new FilePosition(headerFound.StartPosition, lineNumber);
                                    headerEndIndex = headerFound.StartPosition.Index + headerFound.Count - 1;
                                    blockStartIndex = headerFound.StartPosition.Index + headerFound.Count;
                                }

                                break;
                            }
                    }

                    // Find block body
                    FilePosition blockStartPosition = StringHelper.SearchKeywordIndex(
                        trimedLine,
                        identity.ElementBlockOpenSymbol,
                        headerEndIndex + 1,
                        findEndIndex - (headerEndIndex));
                    //FilePosition blockStartPosition = StringHelper.SearchKeywordIndex(
                    //    trimedLine,
                    //    identity.ElementBlockOpenSymbol,
                    //    headerEndIndex + 1,
                    //    endIndex - (headerEndIndex));
                    blockStartPosition.LineNumber = lineNumber;


                    // The block is already started, try to get block body boundary information
                    if (blockStartPosition.IsValidPosition)
                    {
                        // Find block end position
                        FilePosition blockEndPosition = FindBlockEndPosition(identity, trimedLine, blockStartPosition.Index);

                        blockEndPosition.LineNumber = lineNumber;

                        blockFound = new FileElement(identity);

                        // Update block information
                        blockFound.ElementBlockOpenPosition = blockStartPosition;
                        blockFound.ElementBlockClosedPosition = blockEndPosition;
                        blockFound.Header = headerFound;
                    }
                    else if (headerFound != null)
                    {
                        blockFound = new FileElement(identity);
                        blockFound.Header = headerFound;
                    }

                    if (blockFound != null)
                    {
                        break;
                    }
                }

                if (blockFound == null)
                {
                    break;
                }
                else if (!blockFound.ElementBlockClosed)
                {
                    // Add the block to result and exit loop
                    result.Add(blockFound);
                    break;
                }
                else
                {
                    // Add the block to result
                    result.Add(blockFound);
                }

                startIndex = blockFound.ElementBlockClosedPosition.Index + blockFound.ElementBlockClosedPosition.Count;
            }

            return result;
        }

        #endregion

        #region FindElementHeader

        protected FileElementHeader FindNearestElementHeader(String trimedLine, Int32 startIndex, Int32 count, IEnumerable<FileElementIdentity> possibleIdentities)
        {
            FileElementHeader result = null;

            foreach (FileElementIdentity identity in possibleIdentities)
            {
                FileElementHeader headerFound = FindElementHeader(trimedLine, startIndex, count, identity);

                if (headerFound != null)
                {
                    if (result == null || result.StartPosition > headerFound.StartPosition)
                    {
                        result = headerFound;
                    }
                }
            }

            return result;
        }

        protected FileElementHeader FindElementHeader(String trimedLine, Int32 startIndex, Int32 count, FileElementIdentity identity)
        {
            FileElementHeader headerFound = null;

            System.Text.RegularExpressions.Match match = StringHelper.MatchString(trimedLine, identity.HeaderKeyword.Keyword, startIndex, count);

            if (match.Success)
            {
                headerFound = new FileElementHeader();
                headerFound.StartPosition = new FilePosition(match.Index, 1);
                headerFound.Count = match.Length;
                foreach (String headerGroupName in identity.HeaderKeyword.ResultGroupNames.Keys)
                {
                    headerFound.Keywords.Add(identity.HeaderKeyword.ResultGroupNames[headerGroupName], match.Groups[headerGroupName].Value);
                }
            }

            return headerFound;
        }

        private static FileElementHeader CheckFindBlockHeaderResult(FileElementHeader result1, FileElementHeader result2)
        {
            if ((result1 == null) || (((result1 != null) && (result2 != null)) && (result1.StartPosition > result2.StartPosition)))
            {
                return result2;
            }
            return result1;
        }

        #endregion

        #region FindBlockEndPosition

        protected FilePosition FindBlockEndPosition(FileElementIdentity identity, IList<string> lines, FilePosition startPosition, out int remainingLevel, int lineCount)
        {
            FilePosition position = new FilePosition();
            int lineNumber = -1;
            int level = 0;
            bool start = true;

            for (lineNumber = startPosition.LineNumber; lineNumber < lines.Count && (level > 0 || start); lineNumber++)
            {
                String line = lines[lineNumber];

                if (!start)
                {
                    position = FindBlockEndPosition(identity, line, level, out level, 0);
                }
                else
                {
                    position = FindBlockEndPosition(identity, line, level, out level, startPosition.Index);
                    start = false;
                }
            }

            remainingLevel = level;

            if (level >= 0)
            {
                position.LineNumber = lineNumber - 1;

                return position;
            }
            else
            {
                return position;
            }
        }

        protected FilePosition FindBlockEndPosition(FileElementIdentity identity, string line, int currentBlockLevel, out int remainingLevel, int startIndex)
        {
            int start = startIndex;
            int end = startIndex;
            int level = 1;
            int count = line.Length - startIndex;
            int blockEndTokenLength = 0;
            bool isStart = true;
            FilePosition endPosition;

            while (level > 0)
            {
                if (isStart)
                {
                    if (currentBlockLevel > 0)
                    {
                        level = currentBlockLevel;
                    }
                    else
                    {
                        level = 0;
                    }

                    if (count < 0)
                    {
                        end = -1;
                        break;
                    }

                    start = StringHelper.SearchKeywordIndex(line, identity.ElementBlockOpenSymbol, startIndex, count).Index;
                    endPosition = StringHelper.SearchKeywordIndex(line, identity.ElementBlockClosedSymbol, startIndex, count);
                    end = endPosition.Index;

                    // Update end position if start token is the same as end token.
                    if (end == start && end != -1)
                    {
                        if (end + 1 < startIndex + count)
                        {
                            endPosition = StringHelper.SearchKeywordIndex(line, identity.ElementBlockClosedSymbol, startIndex, count);
                            end = endPosition.Index;
                        }
                        else
                        {
                            endPosition.Index = -1;
                            end = -1;
                        }
                    }

                    blockEndTokenLength = endPosition.Count;

                    if (start >= 0 && end < 0)
                    {
                        level++;
                        break;
                    }

                    if (start < 0 && end >= 0)
                    {
                        level--;
                        break;
                    }

                    isStart = false;
                }
                else
                {
                    if (start < end && start >= 0)
                    {
                        start = StringHelper.SearchKeywordIndex(line, identity.ElementBlockOpenSymbol, start + 1, count - (start + 1 - startIndex)).Index;

                        // Update start position if start token is the same as end token.
                        if (end == start && end != -1)
                        {
                            if (start + 1 < startIndex + count)
                            {
                                start = StringHelper.SearchKeywordIndex(line, identity.ElementBlockOpenSymbol, start + 1, count - (start + 1 - startIndex)).Index;
                            }
                            else
                            {
                                start = -1;
                            }
                        }
                    }
                    else
                    {
                        endPosition = StringHelper.SearchKeywordIndex(line, identity.ElementBlockClosedSymbol, end + 1, count - (end + 1 - startIndex));
                        end = endPosition.Index;
                        blockEndTokenLength = endPosition.Count;

                        // Update end position if start token is the same as end token.
                        if (end == start && end != -1)
                        {
                            if (end + 1 < startIndex + count)
                            {
                                endPosition = StringHelper.SearchKeywordIndex(line, identity.ElementBlockClosedSymbol, startIndex, count);
                                end = endPosition.Index;
                            }
                            else
                            {
                                endPosition.Index = -1;
                                end = -1;
                            }
                        }
                    }
                }

                if (end < 0)
                {
                    break;
                }

                if (start < 0 || start > end)
                {
                    level--;
                }
                else
                {
                    level++;
                }
            }

            remainingLevel = level;
            return new FilePosition(end, blockEndTokenLength);
        }

        protected FilePosition FindBlockEndPosition(FileElementIdentity identity, string line, out int remainingLevel, int startIndex)
        {
            return FindBlockEndPosition(identity, line, 0, out remainingLevel, startIndex);
        }

        protected FilePosition FindBlockEndPosition(FileElementIdentity identity, string line, int startIndex)
        {
            int remainingLevel = 0;
            return FindBlockEndPosition(identity, line, out remainingLevel, startIndex);
        }

        #endregion

        private void AppendToOutline(FileElement element)
        {
            if (element.Identity.AppendToOutline)
            {
                if (ElementStack.Count > 0)
                {
                    ElementStack.Peek().Elements.Add(element);
                }
                else
                {
                    Elements.Add(element);
                }
            }
        }

        public FileOutline Generate(TextReader reader)
        {
            FileOutline result = null;

            ParseFile(reader);

            result = new FileOutline(Elements);
            result.StartLine = startLine;
            result.Count = lineCount;

            Reset();

            return result;
        }

        public void Reset()
        {
            _elements.Clear();
            _elementStack.Clear();
            lineCount = 0;
            startLine = 0;
        }
    }
}
