using System;
using System.Collections.Generic;
using System.Text;

namespace LineCounter.Files
{
    public abstract class FileBase : IFile
    {
        private String _name;
        private String _displayName;
        private IFileContainer _container;
        private Boolean _counted = false;
        private FileContentType _contentType;
        private String _address;
        private FileLocation _location;
        private CountResult _counterResult;
        private FileType _type;


        protected FileBase(FileLocation location, FileType type)
        {
            _location = location;
            _type = type;
        }


        #region IFile Members

        public string FullName
        {
            get { return GetFullName(this); }
        }

        public string FullNameDisplayText
        {
            get { return GetFullNameDisplayText(this); }
        }

        public FileContentType ContentType
        {
            get { return _contentType; }
            set { _contentType = value; }
        }

        public virtual IFileContainer Container
        {
            get { return _container; }
            set { _container = value; }
        }

        public string Address
        {
            get { return _address; }
            set { _address = value; }
        }

        public FileLocation Location
        {
            get { return _location; }
            set { _location = value; }
        }

        public FileType Type
        {
            get { return _type; }
        }

        #endregion

        #region INamedObject Members

        public string Name
        {
            get { return _name; }
            set
            {
                if (StringHelper.IsNullOrEmpty(value))
                {
                    throw new ArgumentNullException("value");
                }

                _name = value; 
            }
        }

        public string DisplayName
        {
            get
            {
                if (StringHelper.IsNullOrEmpty(_displayName))
                {
                    return Name;
                }

                return _displayName;
            }
            set
            {
                _displayName = value;
            }
        }

        #endregion

        #region ICountable Members

        public virtual LineCounter.CountResult CounterResult
        {
            get { return _counterResult; }
            set { _counterResult = value; }
        }

        public bool Counted
        {
            get { return _counted; }
            set { _counted = value; }
        }

        public abstract System.IO.StreamReader GetStreamReader();

        #endregion

        #region Helpers

        /// <summary>
        /// Gets the text for displaying the full name of a file.
        /// </summary>
        /// <param name="file">The file.</param>
        /// <returns>The text for displaying full name.</returns>
        /// <exception cref="ArgumentNullException" />
        protected static String GetFullNameDisplayText(IFile file)
        {
            if (file == null)
            {
                throw new ArgumentNullException("file");
            }

            if (file.Container == null)
            {
                return file.DisplayName;
            }
            else
            {
                return System.IO.Path.Combine(file.Container.FullNameDisplayText, file.DisplayName);
            }
        }

        /// <summary>
        /// Gets the full name of a file.
        /// </summary>
        /// <param name="file">The file.</param>
        /// <returns>The file's full name.</returns>
        /// <exception cref="ArgumentNullException" />
        protected static String GetFullName(IFile file)
        {
            if (file == null)
            {
                throw new ArgumentNullException("file");
            }

            if (file.Container == null)
            {
                return file.Name;
            }
            else
            {
                return System.IO.Path.Combine(file.Container.FullName, file.Name);
            }
        }

        #endregion
    }
}
