using System;
using System.Collections.Generic;
using System.Text;

namespace LineCounter.Files
{
    public abstract class FileContainerBase : FileBase, IFileContainer
    {
        private FileDictionary _files;
        private FileContainerDictionary _subContainers;
        private FileContainerType _containerType;


        protected FileContainerBase(FileContainerType containerType, FileLocation location)
            : base(location, FileType.FileContainer)
        {
            _containerType = containerType;
            _files = new FileDictionary();
            _subContainers = new FileContainerDictionary();
        }

        /// <exception cref="NotSupportedException"/>
        public override IFileContainer Container
        {
            get
            {
                if (ContainerType == FileContainerType.Root)
                {
                    return null;
                }
                else
                {
                    return base.Container;
                }
            }
            set
            {
                if (ContainerType == FileContainerType.Root)
                {
                    throw new NotSupportedException(Resources.Resource.GetLocalizedString("Message_AlreadyRoot", this.FullName));
                }
                else
                {
                    base.Container = value;
                }
            }
        }

        #region ICountable Members

        public override CountResult CounterResult
        {
            get
            {
                //if (!Counted)
                //{
                long lines = 0;

                foreach (IFile file in GetFilesByType(FileContentType.Countable, true))
                {
                    if (file.Counted)
                    {
                        lines += file.CounterResult.Lines;
                    }
                }

                foreach (IFileContainer fileContainer in SubContainers.Values)
                {
                    lines += fileContainer.CounterResult.Lines;
                }

                base.CounterResult = new FileContainerCountResult(lines);

                Counted = true;
                //}

                return base.CounterResult;
            }
            set
            {
                throw new NotSupportedException();
            }
        }

        public override System.IO.StreamReader GetStreamReader()
        {
            throw new NotSupportedException();
        }

        #endregion

        #region IFileContainer Members

        public FileDictionary Files
        {
            get { return _files; }
        }

        public FileContainerDictionary SubContainers
        {
            get { return _subContainers; }
        }

        public FileContainerType ContainerType
        {
            get { return _containerType; }
        }

        public IEnumerable<IFile> GetFilesByType(FileContentType type, Boolean topContainerOnly)
        {
            foreach (IFile file in Files.Values)
            {
                if (file.ContentType == type)
                {
                    yield return file;
                }
            }

            if (!topContainerOnly)
            {
                foreach (IFileContainer container in SubContainers.Values)
                {
                    foreach (IFile file in container.GetFilesByType(type, false))
                    {
                        yield return file;
                    }
                }
            }
        }

        public void AddFile(IFile file)
        {
            if (file == null)
            {
                throw new ArgumentNullException("file");
            }

            if (Files.ContainsKey(file.Name) || SubContainers.ContainsKey(file.Name))
            {
                throw new FileExistsException(file.Name, FullNameDisplayText);
            }
            else
            {
                Files.Add(file.Name, file);
                file.Container = this;
            }
        }

        public void AddFileContainer(IFileContainer container)
        {
            if (container == null)
            {
                throw new ArgumentNullException("container");
            }

            if (SubContainers.ContainsKey(container.Name) || Files.ContainsKey(container.Name))
            {
                throw new FileExistsException(container.Name, FullNameDisplayText);
            }
            else
            {
                SubContainers.Add(container.Name, container);
                container.Container = this;
            }
        }

        public void RemoveFile(IFile file)
        {
            if (file == null)
            {
                throw new ArgumentNullException("file");
            }

            file.Container = null;
            Files.Remove(file.Name);
        }

        public void RemoveFileContainer(IFileContainer container)
        {
            if (container == null)
            {
                throw new ArgumentNullException("container");
            }

            container.Container = null;
            SubContainers.Remove(container.Name);
        }

        public Boolean Contains(IFile file)
        {
            if (file == null)
            {
                throw new ArgumentNullException("file");
            }

            if (file.FullName.IndexOf(this.FullName) == 0)
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        public Boolean Contains(IFileContainer container)
        {
            if (container == null)
            {
                throw new ArgumentNullException("container");
            }

            if (container.FullName.IndexOf(this.FullName) == 0)
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        #endregion
    }
}
